@@ -1,21 +1,3 @@
-/****************************************************************************
-                   encoderffmpegcore.cpp  -  FFMPEG encoder for mixxx
-                             -------------------
-    copyright            : (C) 2012-2013 by Tuukka Pasanen
-                           (C) 2007 by Wesley Stessens
-                           (C) 1994 by Xiph.org (encoder example)
-                           (C) 1994 Tobias Rafreider (shoutcast and recording fixes)
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
 //
 //   FFMPEG encoder class..
 //     - Supports what FFMPEG is compiled to supported
@@ -49,10 +31,6 @@ EncoderFfmpegCore::EncoderFfmpegCore(EncoderCallback* pCallback, CodecID codec)
 {
     m_bStreamInitialized = false;
     m_pCallback = pCallback;
-    m_strMetaDataTitle = NULL;
-    m_strMetaDataArtist = NULL;
-    m_strMetaDataAlbum = NULL;
-    m_pMetaData = TrackPointer(NULL);
 
     m_pEncodeFormatCtx = NULL;
     m_pEncoderAudioStream = NULL;
@@ -109,7 +87,12 @@ EncoderFfmpegCore::~EncoderFfmpegCore() {
     delete m_pResample;
 }
 
-//call sendPackages() or write() after 'flush()' as outlined in engineshoutcast.cpp
+void EncoderFfmpegCore::setEncoderSettings(const EncoderSettings& settings)
+{
+    m_lBitrate = settings.getQuality() * 1000;
+}
+
+//call sendPackages() or write() after 'flush()' as outlined in enginebroadcast.cpp
 void EncoderFfmpegCore::flush() {
 }
 
@@ -207,12 +190,12 @@ void EncoderFfmpegCore::encodeBuffer(const CSAMPLE *samples, const int size) {
     free(l_fNormalizedSamples);
 }
 
-// Originally called from engineshoutcast.cpp to update metadata information
+// Originally called from enginebroadcast.cpp to update metadata information
 // when streaming, however, this causes pops
 //
 // Currently this method is used before init() once to save artist, title and album
 //
-void EncoderFfmpegCore::updateMetaData(char* artist, char* title, char* album) {
+void EncoderFfmpegCore::updateMetaData(const QString& artist, const QString& title, const QString& album) {
     qDebug() << "ffmpegencodercore: UpdateMetadata: !" << artist << " - " << title <<
              " - " << album;
     m_strMetaDataTitle = title;
@@ -220,15 +203,15 @@ void EncoderFfmpegCore::updateMetaData(char* artist, char* title, char* album) {
     m_strMetaDataAlbum = album;
 }
 
-int EncoderFfmpegCore::initEncoder(int bitrate, int samplerate) {
+int EncoderFfmpegCore::initEncoder(int samplerate, QString errorMessage) {
 
 #ifndef avformat_alloc_output_context2
     qDebug() << "EncoderFfmpegCore::initEncoder: Old Style initialization";
     m_pEncodeFormatCtx = avformat_alloc_context();
 #endif
 
-    m_lBitrate = bitrate * 1000;
     m_lSampleRate = samplerate;
+    QString codecString;
 
 #if LIBAVCODEC_VERSION_INT > 3544932
     if (m_SCcodecId == AV_CODEC_ID_MP3) {
@@ -236,6 +219,7 @@ int EncoderFfmpegCore::initEncoder(int bitrate, int samplerate) {
     if (m_SCcodecId == CODEC_ID_MP3) {
 #endif // LIBAVCODEC_VERSION_INT > 3544932
         qDebug() << "EncoderFfmpegCore::initEncoder: Codec MP3";
+        codecString = "Codec MP3";
 #ifdef avformat_alloc_output_context2
         avformat_alloc_output_context2(&m_pEncodeFormatCtx, NULL, NULL, "output.mp3");
 #else
@@ -248,6 +232,7 @@ int EncoderFfmpegCore::initEncoder(int bitrate, int samplerate) {
     } else if (m_SCcodecId == CODEC_ID_AAC) {
 #endif // LIBAVCODEC_VERSION_INT > 3544932
         qDebug() << "EncoderFfmpegCore::initEncoder: Codec M4A";
+        codecString = "Codec M4A";
 #ifdef avformat_alloc_output_context2
         avformat_alloc_output_context2(&m_pEncodeFormatCtx, NULL, NULL, "output.m4a");
 #else
@@ -256,6 +241,7 @@ int EncoderFfmpegCore::initEncoder(int bitrate, int samplerate) {
 
     } else {
         qDebug() << "EncoderFfmpegCore::initEncoder: Codec OGG/Vorbis";
+        codecString = "Codec OGG/Vorbis";
 #ifdef avformat_alloc_output_context2
         avformat_alloc_output_context2(&m_pEncodeFormatCtx, NULL, NULL, "output.ogg");
         m_pEncodeFormatCtx->oformat->audio_codec=AV_CODEC_ID_VORBIS;
@@ -278,11 +264,15 @@ int EncoderFfmpegCore::initEncoder(int bitrate, int samplerate) {
     m_pEncoderAudioStream = addStream(m_pEncodeFormatCtx, &m_pEncoderAudioCodec,
                                       m_pEncoderFormat->audio_codec);
 
-    openAudio(m_pEncoderAudioCodec, m_pEncoderAudioStream);
+    int ret = openAudio(m_pEncoderAudioCodec, m_pEncoderAudioStream);
 
-    // qDebug() << "jepusti";
+    if (ret != 0) {
+        errorMessage  = codecString + "recording is not supported. FFMPEG " 
+                        + codecString + " could not be initialized";
+        ret = -1;
+    };
 
-    return 0;
+    return ret;
 }
 
 // Private methods
@@ -291,7 +281,11 @@ int EncoderFfmpegCore::writeAudioFrame(AVFormatContext *formatctx,
                                        AVStream *stream) {
     AVCodecContext *l_SCodecCtx = NULL;;
     AVPacket l_SPacket;
+#if LIBAVCODEC_VERSION_INT < 3617792
     AVFrame *l_SFrame = avcodec_alloc_frame();
+#else
+    AVFrame *l_SFrame = av_frame_alloc();
+#endif
     int l_iGotPacket;
     int l_iRet;
     uint8_t *l_iOut = NULL;
@@ -341,16 +335,27 @@ int EncoderFfmpegCore::writeAudioFrame(AVFormatContext *formatctx,
     // to something that fits..
     if (l_SCodecCtx->sample_fmt != AV_SAMPLE_FMT_FLT) {
 
-        m_pResample->reSample(l_SFrame, &l_iOut);
+        m_pResample->reSampleMixxx(l_SFrame, &l_iOut);
         // After we have turned our samples to destination
         // Format we must re-alloc l_SFrame.. it easier like this..
-#if LIBAVCODEC_VERSION_INT > 3544932
-        avcodec_free_frame(&l_SFrame);
-#else
+        // FFMPEG 2.2 3561060 anb beyond
+#if LIBAVCODEC_VERSION_INT >= 3561060
+        av_frame_unref(l_SFrame);
+        av_frame_free(&l_SFrame);
+// FFMPEG 0.11 and below
+#elif LIBAVCODEC_VERSION_INT <= 3544932
         av_free(l_SFrame);
-#endif // LIBAVCODEC_VERSION_INT > 3544932
+// FFMPEG 1.0 - 2.1
+#else
+        avcodec_free_frame(&l_SFrame);
+#endif
         l_SFrame = NULL;
+
+#if LIBAVCODEC_VERSION_INT < 3617792
         l_SFrame = avcodec_alloc_frame();
+#else
+        l_SFrame = av_frame_alloc();
+#endif
         l_SFrame->nb_samples = m_iAudioInputFrameSize;
         l_SFrame->format = l_SCodecCtx->sample_fmt;
 #ifndef __FFMPEGOLDAPI__
@@ -403,12 +408,14 @@ int EncoderFfmpegCore::writeAudioFrame(AVFormatContext *formatctx,
     l_SPacket.dts = m_lDts;
     l_SPacket.pts = m_lDts;
 
+#if LIBAVCODEC_VERSION_INT < 3617792
     // Some times den is zero.. so 0 dived by 0 is
     // Something?
     if (m_pEncoderAudioStream->pts.den == 0) {
         qDebug() << "Time hack!";
         m_pEncoderAudioStream->pts.den = 1;
     }
+#endif
 
     // Write the compressed frame to the media file. */
     l_iRet = av_interleaved_write_frame(formatctx, &l_SPacket);
@@ -419,7 +426,11 @@ int EncoderFfmpegCore::writeAudioFrame(AVFormatContext *formatctx,
     }
 
     av_free_packet(&l_SPacket);
+#if LIBAVCODEC_VERSION_INT < 3617792
     av_destruct_packet(&l_SPacket);
+#else
+    av_free_packet(&l_SPacket);
+#endif
     av_free(l_SFrame);
 
     return 0;
@@ -431,7 +442,7 @@ void EncoderFfmpegCore::closeAudio(AVStream *stream) {
     av_free(m_pSamples);
 }
 
-void EncoderFfmpegCore::openAudio(AVCodec *codec, AVStream *stream) {
+int EncoderFfmpegCore::openAudio(AVCodec *codec, AVStream *stream) {
     AVCodecContext *l_SCodecCtx;
     int l_iRet;
 
@@ -443,7 +454,7 @@ void EncoderFfmpegCore::openAudio(AVCodec *codec, AVStream *stream) {
     l_iRet = avcodec_open2(l_SCodecCtx, codec, NULL);
     if (l_iRet < 0) {
         qDebug() << "Could not open audio codec!";
-        return;
+        return -1;
     }
 
     if (l_SCodecCtx->codec->capabilities & CODEC_CAP_VARIABLE_FRAME_SIZE) {
@@ -467,11 +478,10 @@ void EncoderFfmpegCore::openAudio(AVCodec *codec, AVStream *stream) {
 
     if (!m_pSamples) {
         qDebug() << "Could not allocate audio samples buffer";
-        return;
+        return -2;
     }
 
-
-
+    return 0;
 }
 
 // Add an output stream.
@@ -514,7 +524,7 @@ AVStream *EncoderFfmpegCore::addStream(AVFormatContext *formatctx,
         l_SCodecCtx->sample_rate = 44100;
         l_SCodecCtx->channels    = 2;
 
-        m_pResample->open(AV_SAMPLE_FMT_FLT, l_SCodecCtx->sample_fmt);
+        m_pResample->openMixxx(AV_SAMPLE_FMT_FLT, l_SCodecCtx->sample_fmt);
         break;
 
     default:
