@@ -2,16 +2,16 @@
 // Created 11/5/2009 by RJ Ryan (rryan@mit.edu)
 
 #include <QMutexLocker>
+#include <QStringBuilder>
 
+#include "engine/enginebuffer.h"
 #include "engine/cuecontrol.h"
 
-#include "controlobject.h"
-#include "controlpushbutton.h"
-#include "controlindicator.h"
-#include "trackinfoobject.h"
-#include "library/dao/cue.h"
-#include "cachingreader.h"
+#include "control/controlobject.h"
+#include "control/controlpushbutton.h"
+#include "control/controlindicator.h"
 #include "vinylcontrol/defs_vinylcontrol.h"
+#include "util/sample.h"
 
 // TODO: Convert these doubles to a standard enum
 // and convert elseif logic to switch statements
@@ -20,12 +20,15 @@ static const double CUE_MODE_PIONEER = 1.0;
 static const double CUE_MODE_DENON = 2.0;
 static const double CUE_MODE_NUMARK = 3.0;
 static const double CUE_MODE_MIXXX_NO_BLINK = 4.0;
+static const double CUE_MODE_CUP = 5.0;
 
 CueControl::CueControl(QString group,
-                       ConfigObject<ConfigValue>* _config) :
-        EngineControl(group, _config),
+                       UserSettingsPointer pConfig) :
+        EngineControl(group, pConfig),
         m_bPreviewing(false),
-        m_pPlayButton(ControlObject::getControl(ConfigKey(group, "play"))),
+        // m_pPlay->toBoo() -> engine play state
+        // m_pPlay->set(1.0) -> emulate play button press
+        m_pPlay(ControlObject::getControl(ConfigKey(group, "play"))),
         m_pStopButton(ControlObject::getControl(ConfigKey(group, "stop"))),
         m_iCurrentlyPreviewingHotcues(0),
         m_bypassCueSetByPlay(false),
@@ -65,6 +68,12 @@ CueControl::CueControl(QString group,
             this, SLOT(cueGotoAndPlay(double)),
             Qt::DirectConnection);
 
+    m_pCuePlay =
+            new ControlPushButton(ConfigKey(group, "cue_play"));
+    connect(m_pCuePlay, SIGNAL(valueChanged(double)),
+            this, SLOT(cuePlay(double)),
+            Qt::DirectConnection);
+
     m_pCueGotoAndStop =
             new ControlPushButton(ConfigKey(group, "cue_gotoandstop"));
     connect(m_pCueGotoAndStop, SIGNAL(valueChanged(double)),
@@ -94,8 +103,8 @@ CueControl::CueControl(QString group,
     m_pCueIndicator = new ControlIndicator(ConfigKey(group, "cue_indicator"));
     m_pPlayIndicator = new ControlIndicator(ConfigKey(group, "play_indicator"));
 
-    m_pVinylControlEnabled = new ControlObjectSlave(group, "vinylcontrol_enabled");
-    m_pVinylControlMode = new ControlObjectSlave(group, "vinylcontrol_mode");
+    m_pVinylControlEnabled = new ControlProxy(group, "vinylcontrol_enabled");
+    m_pVinylControlMode = new ControlProxy(group, "vinylcontrol_mode");
 }
 
 CueControl::~CueControl() {
@@ -104,6 +113,7 @@ CueControl::~CueControl() {
     delete m_pCueSet;
     delete m_pCueGoto;
     delete m_pCueGotoAndPlay;
+    delete m_pCuePlay;
     delete m_pCueGotoAndStop;
     delete m_pCuePreview;
     delete m_pCueCDJ;
@@ -113,7 +123,7 @@ CueControl::~CueControl() {
     delete m_pPlayIndicator;
     delete m_pVinylControlEnabled;
     delete m_pVinylControlMode;
-    qDeleteAll(m_hotcueControl);
+    qDeleteAll(m_hotcueControls);
 }
 
 void CueControl::createControls() {
@@ -145,96 +155,98 @@ void CueControl::createControls() {
                 this, SLOT(hotcueClear(HotcueControl*, double)),
                 Qt::DirectConnection);
 
-        m_hotcueControl.append(pControl);
+        m_hotcueControls.append(pControl);
     }
 }
 
-void CueControl::attachCue(Cue* pCue, int hotCue) {
-    HotcueControl* pControl = m_hotcueControl.value(hotCue, NULL);
+void CueControl::attachCue(CuePointer pCue, int hotCue) {
+    HotcueControl* pControl = m_hotcueControls.value(hotCue, NULL);
     if (pControl == NULL) {
         return;
     }
     if (pControl->getCue() != NULL) {
         detachCue(pControl->getHotcueNumber());
     }
-    connect(pCue, SIGNAL(updated()),
+    connect(pCue.get(), SIGNAL(updated()),
             this, SLOT(cueUpdated()),
             Qt::DirectConnection);
 
-    pControl->getPosition()->set(pCue->getPosition());
-    pControl->getEnabled()->set(pCue->getPosition() == -1 ? 0.0 : 1.0);
-    // set pCue only if all other data is in place
-    // because we have a null check for valid data else where  in the code
     pControl->setCue(pCue);
 
 }
 
 void CueControl::detachCue(int hotCue) {
-    HotcueControl* pControl = m_hotcueControl.value(hotCue, NULL);
+    HotcueControl* pControl = m_hotcueControls.value(hotCue, NULL);
     if (pControl == NULL) {
         return;
     }
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
     if (!pCue)
         return;
-    disconnect(pCue, 0, this, 0);
-    // clear pCue first because we have a null check for valid data else where
-    // in the code
-    pControl->setCue(NULL);
-    pControl->getPosition()->set(-1); // invalidate position for hintReader()
-    pControl->getEnabled()->set(0);
+    disconnect(pCue.get(), 0, this, 0);
+    pControl->resetCue();
 }
 
-void CueControl::trackLoaded(TrackPointer pTrack) {
+void CueControl::trackLoaded(TrackPointer pNewTrack, TrackPointer pOldTrack) {
     QMutexLocker lock(&m_mutex);
-    if (m_pLoadedTrack)
-        trackUnloaded(m_pLoadedTrack);
 
-    if (!pTrack) {
-        return;
+    DEBUG_ASSERT(m_pLoadedTrack == pOldTrack);
+    if (m_pLoadedTrack) {
+        disconnect(m_pLoadedTrack.get(), 0, this, 0);
+        for (int i = 0; i < m_iNumHotCues; ++i) {
+            detachCue(i);
+        }
+
+        m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+        m_pCuePoint->set(-1.0);
+        m_pLoadedTrack.reset();
     }
 
-    m_pLoadedTrack = pTrack;
-    connect(pTrack.data(), SIGNAL(cuesUpdated()),
+    if (!pNewTrack) {
+        return;
+    }
+    m_pLoadedTrack = pNewTrack;
+
+    connect(m_pLoadedTrack.get(), SIGNAL(cuesUpdated()),
             this, SLOT(trackCuesUpdated()),
             Qt::DirectConnection);
 
-    Cue* loadCue = NULL;
-    const QList<Cue*>& cuePoints = pTrack->getCuePoints();
-    QListIterator<Cue*> it(cuePoints);
-    while (it.hasNext()) {
-        Cue* pCue = it.next();
+    CuePointer pLoadCue;
+    for (const CuePointer& pCue: m_pLoadedTrack->getCuePoints()) {
+        if (pCue->getType() == Cue::CUE) {
+            continue; // skip
+        }
         if (pCue->getType() == Cue::LOAD) {
-            loadCue = pCue;
-        } else if (pCue->getType() != Cue::CUE) {
-            continue;
+            DEBUG_ASSERT(!pLoadCue);
+            pLoadCue = pCue;
         }
         int hotcue = pCue->getHotCue();
-        if (hotcue != -1)
+        if (hotcue != -1) {
             attachCue(pCue, hotcue);
-    }
-
-    double loadCuePoint = 0.0;
-    // If cue recall is ON in the prefs, then we're supposed to seek to the cue
-    // point on song load. Note that [Controls],cueRecall == 0 corresponds to "ON", not OFF.
-    bool cueRecall = (getConfig()->getValueString(
-                ConfigKey("[Controls]","CueRecall"), "0").toInt() == 0);
-    if (loadCue != NULL) {
-        m_pCuePoint->set(loadCue->getPosition());
-        if (cueRecall) {
-            loadCuePoint = loadCue->getPosition();
         }
-    } else {
-        // If no cue point is stored, set one at track start
-        m_pCuePoint->set(0.0);
     }
+    double cuePoint;
+    if (pLoadCue) {
+        cuePoint = pLoadCue->getPosition();
+    } else {
+        // If no load cue point is stored, read from track
+        cuePoint = m_pLoadedTrack->getCuePoint();
+    }
+    m_pCuePoint->set(cuePoint);
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
-    // If cueRecall is on, seek to it even if we didn't find a cue value (we'll
-    // seek to 0.
-    if (cueRecall) {
-        seekExact(loadCuePoint);
+
+    // Use pNewTrack here, because m_pLoadedTrack might have been reset
+    // immediately after leaving the locking scope!
+    pNewTrack->setCuePoint(cuePoint);
+
+    // If cue recall is ON in the prefs, then we're supposed to seek to the cue
+    // point on song load. Note that [Controls],cueRecall == 0 corresponds to "ON", not OFF.
+    bool cueRecall = (getConfig()->getValue(
+           ConfigKey("[Controls]","CueRecall"), 0) == 0);
+    if (cueRecall && (cuePoint >= 0.0)) {
+        seekExact(cuePoint);
     } else if (!(m_pVinylControlEnabled->get() &&
             m_pVinylControlMode->get() == MIXXX_VCMODE_ABSOLUTE)) {
         // If cuerecall is off, seek to zero unless
@@ -242,40 +254,8 @@ void CueControl::trackLoaded(TrackPointer pTrack) {
         // load tracks and have the needle-drop be maintained.
         seekExact(0.0);
     }
-}
 
-void CueControl::trackUnloaded(TrackPointer pTrack) {
-    QMutexLocker lock(&m_mutex);
-    disconnect(pTrack.data(), 0, this, 0);
-    for (int i = 0; i < m_iNumHotCues; ++i) {
-        detachCue(i);
-    }
-
-    // Store the cue point in a load cue.
-    double cuePoint = m_pCuePoint->get();
-
-    if (cuePoint != -1 && cuePoint != 0.0) {
-        Cue* loadCue = NULL;
-        const QList<Cue*>& cuePoints = pTrack->getCuePoints();
-        QListIterator<Cue*> it(cuePoints);
-        while (it.hasNext()) {
-            Cue* pCue = it.next();
-            if (pCue->getType() == Cue::LOAD) {
-                loadCue = pCue;
-                break;
-            }
-        }
-        if (!loadCue) {
-            loadCue = pTrack->addCue();
-            loadCue->setType(Cue::LOAD);
-            loadCue->setLength(0);
-        }
-        loadCue->setPosition(cuePoint);
-    }
-
-    m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
-    m_pCuePoint->set(-1.0);
-    m_pLoadedTrack.clear();
+    trackCuesUpdated();
 }
 
 void CueControl::cueUpdated() {
@@ -290,43 +270,35 @@ void CueControl::trackCuesUpdated() {
     if (!m_pLoadedTrack)
         return;
 
-    const QList<Cue*>& cuePoints = m_pLoadedTrack->getCuePoints();
-    QListIterator<Cue*> it(cuePoints);
+    const QList<CuePointer> cuePoints(m_pLoadedTrack->getCuePoints());
+    QListIterator<CuePointer> it(cuePoints);
     while (it.hasNext()) {
-        Cue* pCue = it.next();
+        CuePointer pCue(it.next());
 
         if (pCue->getType() != Cue::CUE && pCue->getType() != Cue::LOAD)
             continue;
 
         int hotcue = pCue->getHotCue();
         if (hotcue != -1) {
-            HotcueControl* pControl = m_hotcueControl.value(hotcue, NULL);
+            HotcueControl* pControl = m_hotcueControls.value(hotcue, NULL);
 
             // Cue's hotcue doesn't have a hotcue control.
             if (pControl == NULL) {
                 continue;
             }
 
-            Cue* pOldCue = pControl->getCue();
+            CuePointer pOldCue(pControl->getCue());
 
             // If the old hotcue is different than this one.
             if (pOldCue != pCue) {
                 // If the old hotcue exists, detach it
-                if (pOldCue != NULL)
+                if (pOldCue) {
                     detachCue(hotcue);
+                }
                 attachCue(pCue, hotcue);
             } else {
                 // If the old hotcue is the same, then we only need to update
-                double dOldPosition = pControl->getPosition()->get();
-                double dOldEnabled = pControl->getEnabled()->get();
-                double dPosition = pCue->getPosition();
-                double dEnabled = dPosition == -1 ? 0.0 : 1.0;
-                if (dEnabled != dOldEnabled) {
-                    pControl->getEnabled()->set(dEnabled);
-                }
-                if (dPosition != dOldPosition) {
-                    pControl->getPosition()->set(dPosition);
-                }
+                pControl->setPosition(pCue->getPosition());
             }
             // Add the hotcue to the list of active hotcues
             active_hotcues.insert(hotcue);
@@ -335,8 +307,9 @@ void CueControl::trackCuesUpdated() {
 
     // Detach all hotcues that are no longer present
     for (int i = 0; i < m_iNumHotCues; ++i) {
-        if (!active_hotcues.contains(i))
+        if (!active_hotcues.contains(i)) {
             detachCue(i);
+        }
     }
 }
 
@@ -351,13 +324,17 @@ void CueControl::hotcueSet(HotcueControl* pControl, double v) {
         return;
 
     int hotcue = pControl->getHotcueNumber();
-    detachCue(hotcue);
-    Cue* pCue = m_pLoadedTrack->addCue();
+    // Note: the cue is just detached from the hotcue control
+    // It remains in the database for later use
+    // TODO: find a rule, that allows us to delete the cue as well
+    // https://bugs.launchpad.net/mixxx/+bug/1653276
+    hotcueClear(pControl, v);
+
+    CuePointer pCue(m_pLoadedTrack->createAndAddCue());
+    double closestBeat = m_pClosestBeat->get();
     double cuePosition =
-            (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-            floor(m_pClosestBeat->get()) : floor(getCurrentSample());
-    if (!even(static_cast<int>(cuePosition)))
-        cuePosition--;
+            (m_pQuantizeEnabled->toBool() && closestBeat != -1) ?
+                    closestBeat : getCurrentSample();
     pCue->setPosition(cuePosition);
     pCue->setHotCue(hotcue);
     pCue->setLabel("");
@@ -368,7 +345,7 @@ void CueControl::hotcueSet(HotcueControl* pControl, double v) {
     // If quantize is enabled and we are not playing, jump to the cue point
     // since it's not necessarily where we currently are. TODO(XXX) is this
     // potentially invalid for vinyl control?
-    bool playing = m_pPlayButton->toBool();
+    bool playing = m_pPlay->toBool();
     if (!playing && m_pQuantizeEnabled->get() > 0.0) {
         lock.unlock();  // prevent deadlock.
         // Enginebuffer will quantize more exactly than we can.
@@ -385,7 +362,7 @@ void CueControl::hotcueGoto(HotcueControl* pControl, double v) {
         return;
     }
 
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
@@ -406,7 +383,7 @@ void CueControl::hotcueGotoAndStop(HotcueControl* pControl, double v) {
     if (!m_pLoadedTrack)
         return;
 
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
@@ -414,7 +391,7 @@ void CueControl::hotcueGotoAndStop(HotcueControl* pControl, double v) {
     if (pCue) {
         int position = pCue->getPosition();
         if (position != -1) {
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
             seekExact(position);
         }
     }
@@ -429,7 +406,7 @@ void CueControl::hotcueGotoAndPlay(HotcueControl* pControl, double v) {
         return;
     }
 
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
 
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
@@ -438,9 +415,15 @@ void CueControl::hotcueGotoAndPlay(HotcueControl* pControl, double v) {
         int position = pCue->getPosition();
         if (position != -1) {
             seekAbs(position);
-            // don't move the cue point to the hot cue point in DENON mode
-            m_bypassCueSetByPlay = true;
-            m_pPlayButton->set(1.0);
+            if (!isPlayingByPlayButton()) {
+                // cueGoto is processed asynchrony.
+                // avoid a wrong cue set if seek by cueGoto is still pending
+                m_bPreviewing = false;
+                m_iCurrentlyPreviewingHotcues = 0;
+                // don't move the cue point to the hot cue point in DENON mode
+                m_bypassCueSetByPlay = true;
+                m_pPlay->set(1.0);
+            }
         }
     }
 }
@@ -454,7 +437,7 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
         return;
     }
 
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
 
     lock.unlock();
 
@@ -463,9 +446,7 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
             if (pCue->getPosition() == -1) {
                 hotcueSet(pControl, v);
             } else {
-                if (!m_iCurrentlyPreviewingHotcues &&
-                        !m_bPreviewing &&
-                        m_pPlayButton->toBool()) {
+                if (isPlayingByPlayButton()) {
                     hotcueGoto(pControl, v);
                 } else {
                     hotcueActivatePreview(pControl, v);
@@ -495,14 +476,14 @@ void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
     if (!m_pLoadedTrack) {
         return;
     }
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
 
     if (v) {
         if (pCue && pCue->getPosition() != -1) {
             m_iCurrentlyPreviewingHotcues++;
             int iPosition = pCue->getPosition();
             m_bypassCueSetByPlay = true;
-            m_pPlayButton->set(1.0);
+            m_pPlay->set(1.0);
             pControl->setPreviewing(true);
             pControl->setPreviewingPosition(iPosition);
 
@@ -522,7 +503,7 @@ void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
 
             // If this is the last hotcue to leave preview.
             if (--m_iCurrentlyPreviewingHotcues == 0 && !m_bPreviewing) {
-                m_pPlayButton->set(0.0);
+                m_pPlay->set(0.0);
                 // Need to unlock before emitting any signals to prevent deadlock.
                 lock.unlock();
                 seekExact(iPosition);
@@ -540,12 +521,9 @@ void CueControl::hotcueClear(HotcueControl* pControl, double v) {
         return;
     }
 
-    Cue* pCue = pControl->getCue();
-    if (pCue) {
-        pCue->setHotCue(-1);
-    }
-
+    CuePointer pCue(pControl->getCue());
     detachCue(pControl->getHotcueNumber());
+    m_pLoadedTrack->removeCue(pCue);
 }
 
 void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPosition) {
@@ -553,19 +531,14 @@ void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPositi
     if (!m_pLoadedTrack)
         return;
 
-    Cue* pCue = pControl->getCue();
+    CuePointer pCue(pControl->getCue());
     if (pCue) {
         // Setting the position to -1 is the same as calling hotcue_x_clear
         if (newPosition == -1) {
             pCue->setHotCue(-1);
             detachCue(pControl->getHotcueNumber());
         } else if (newPosition > 0 && newPosition < m_pTrackSamples->get()) {
-            int position = newPosition;
-            // People writing from MIDI land, elsewhere might be careless.
-            if (position % 2 != 0) {
-                position--;
-            }
-            pCue->setPosition(position);
+            pCue->setPosition(newPosition);
         }
     }
 }
@@ -574,8 +547,8 @@ void CueControl::hintReader(HintVector* pHintList) {
     Hint cue_hint;
     double cuePoint = m_pCuePoint->get();
     if (cuePoint >= 0) {
-        cue_hint.sample = m_pCuePoint->get();
-        cue_hint.length = 0;
+        cue_hint.frame = SampleUtil::floorPlayPosToFrame(m_pCuePoint->get());
+        cue_hint.frameCount = Hint::kFrameCountForward;
         cue_hint.priority = 10;
         pHintList->append(cue_hint);
     }
@@ -583,27 +556,17 @@ void CueControl::hintReader(HintVector* pHintList) {
     // this is called from the engine thread
     // it is no locking required, because m_hotcueControl is filled during the
     // constructor and getPosition()->get() is a ControlObject
-    for (QList<HotcueControl*>::const_iterator it = m_hotcueControl.constBegin();
-         it != m_hotcueControl.constEnd(); ++it) {
-        HotcueControl* pControl = *it;
-        double position = pControl->getPosition()->get();
+    for (const auto& pControl: m_hotcueControls) {
+        double position = pControl->getPosition();
         if (position != -1) {
-            cue_hint.sample = position;
-            if (cue_hint.sample % 2 != 0)
-                cue_hint.sample--;
-            cue_hint.length = 0;
+            cue_hint.frame = SampleUtil::floorPlayPosToFrame(position);
+            cue_hint.frameCount = Hint::kFrameCountForward;
             cue_hint.priority = 10;
             pHintList->append(cue_hint);
         }
     }
 }
 
-void CueControl::saveCuePoint(double cuePoint) {
-    if (m_pLoadedTrack) {
-        m_pLoadedTrack->setCuePoint(cuePoint);
-    }
-}
-
 // Moves the cue point to current position or to closest beat in case
 // quantize is enabled
 void CueControl::cueSet(double v) {
@@ -611,12 +574,17 @@ void CueControl::cueSet(double v) {
         return;
 
     QMutexLocker lock(&m_mutex);
-    double cue = (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-            floor(m_pClosestBeat->get()) : floor(getCurrentSample());
-    if (!even(static_cast<int>(cue)))
-        cue--;
+    double closestBeat = m_pClosestBeat->get();
+    double cue = (m_pQuantizeEnabled->get() > 0.0 && closestBeat != -1) ?
+            closestBeat : getCurrentSample();
     m_pCuePoint->set(cue);
-    saveCuePoint(cue);
+    TrackPointer pLoadedTrack = m_pLoadedTrack;
+    lock.unlock();
+
+    // Store cue point in loaded track
+    if (pLoadedTrack) {
+        pLoadedTrack->setCuePoint(cue);
+    }
 }
 
 void CueControl::cueGoto(double v)
@@ -636,16 +604,17 @@ void CueControl::cueGoto(double v)
 
 void CueControl::cueGotoAndPlay(double v)
 {
-    if (!v)
-        return;
+    if (!v) return;
     cueGoto(v);
     QMutexLocker lock(&m_mutex);
     // Start playing if not already
-    if (!m_pPlayButton->toBool()) {
+    if (!isPlayingByPlayButton()) {
         // cueGoto is processed asynchrony.
         // avoid a wrong cue set if seek by cueGoto is still pending
+        m_bPreviewing = false;
+        m_iCurrentlyPreviewingHotcues = 0;
         m_bypassCueSetByPlay = true;
-        m_pPlayButton->set(1.0);
+        m_pPlay->set(1.0);
     }
 }
 
@@ -655,7 +624,7 @@ void CueControl::cueGotoAndStop(double v)
         return;
 
     QMutexLocker lock(&m_mutex);
-    m_pPlayButton->set(0.0);
+    m_pPlay->set(0.0);
     double cuePoint = m_pCuePoint->get();
 
     // Need to unlock before emitting any signals to prevent deadlock.
@@ -671,11 +640,11 @@ void CueControl::cuePreview(double v)
     if (v) {
         m_bPreviewing = true;
         m_bypassCueSetByPlay = true;
-        m_pPlayButton->set(1.0);
+        m_pPlay->set(1.0);
     } else if (!v && m_bPreviewing) {
         m_bPreviewing = false;
         if (!m_iCurrentlyPreviewingHotcues) {
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
         } else {
             return;
         }
@@ -691,13 +660,13 @@ void CueControl::cuePreview(double v)
 
 void CueControl::cueCDJ(double v) {
     // This is how Pioneer cue buttons work:
-    // If pressed while playing, stop playback and go to cue.
+    // If pressed while freely playing (i.e. playing and platter NOT being touched), stop playback and go to cue.
+    // If pressed while NOT freely playing (i.e. stopped or playing but platter IS being touched), set new cue point.
     // If pressed while stopped and at cue, play while pressed.
-    // If pressed while stopped and not at cue, set new cue point.
     // If play is pressed while holding cue, the deck is now playing. (Handled in playFromCuePreview().)
 
     QMutexLocker lock(&m_mutex);
-    bool playing = m_pPlayButton->toBool();
+    const auto freely_playing = m_pPlay->toBool() && !getEngineBuffer()->getScratching();
 
     if (v) {
         if (m_iCurrentlyPreviewingHotcues) {
@@ -706,12 +675,12 @@ void CueControl::cueCDJ(double v) {
             m_bPreviewing = true;
             lock.unlock();
             seekAbs(m_pCuePoint->get());
-        } else if (playing || atEndPosition()) {
+        } else if (freely_playing || atEndPosition()) {
             // Jump to cue when playing or when at end position
 
             // Just in case.
             m_bPreviewing = false;
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
@@ -720,12 +689,13 @@ void CueControl::cueCDJ(double v) {
         } else if (isTrackAtCue()) {
             // pause at cue point
             m_bPreviewing = true;
-            m_pPlayButton->set(1.0);
+            m_pPlay->set(1.0);
         } else {
             // Pause not at cue point and not at end position
             cueSet(v);
             // Just in case.
             m_bPreviewing = false;
+            m_pPlay->set(0.0);
 
             // If quantize is enabled, jump to the cue point since it's not
             // necessarily where we currently are
@@ -738,7 +708,7 @@ void CueControl::cueCDJ(double v) {
     } else if (m_bPreviewing) {
         m_bPreviewing = false;
         if (!m_iCurrentlyPreviewingHotcues) {
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
@@ -748,7 +718,7 @@ void CueControl::cueCDJ(double v) {
     }
     // indicator may flash because the delayed adoption of seekAbs
     // Correct the Indicator set via play
-    if (m_pLoadedTrack && !playing) {
+    if (m_pLoadedTrack && !freely_playing) {
         m_pCueIndicator->setBlinkValue(ControlIndicator::ON);
     } else {
         m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
@@ -762,7 +732,7 @@ void CueControl::cueDenon(double v) {
     // Cue Point is moved by play from pause
 
     QMutexLocker lock(&m_mutex);
-    bool playing = (m_pPlayButton->toBool());
+    bool playing = (m_pPlay->toBool());
 
     if (v) {
         if (m_iCurrentlyPreviewingHotcues) {
@@ -774,11 +744,11 @@ void CueControl::cueDenon(double v) {
         } else if (!playing && isTrackAtCue()) {
             // pause at cue point
             m_bPreviewing = true;
-            m_pPlayButton->set(1.0);
+            m_pPlay->set(1.0);
         } else {
             // Just in case.
             m_bPreviewing = false;
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
@@ -788,7 +758,7 @@ void CueControl::cueDenon(double v) {
     } else if (m_bPreviewing) {
         m_bPreviewing = false;
         if (!m_iCurrentlyPreviewingHotcues) {
-            m_pPlayButton->set(0.0);
+            m_pPlay->set(0.0);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
@@ -798,11 +768,55 @@ void CueControl::cueDenon(double v) {
     }
 }
 
+void CueControl::cuePlay(double v) {
+    // This is how CUP button works:
+    // If freely playing (i.e. playing and platter NOT being touched), press to go to cue and stop.
+    // If not freely playing (i.e. stopped or platter IS being touched), press to go to cue and stop.
+    // On release, start playing from cue point.
+
+
+    QMutexLocker lock(&m_mutex);
+    const auto freely_playing = m_pPlay->toBool() && !getEngineBuffer()->getScratching();
+
+    // pressed
+    if (v) {
+        if (freely_playing) {
+            m_bPreviewing = false;
+            m_pPlay->set(0.0);
+
+            // Need to unlock before emitting any signals to prevent deadlock.
+            lock.unlock();
+
+            seekAbs(m_pCuePoint->get());
+        } else if (!isTrackAtCue() && getCurrentSample() <= getTotalSamples()) {
+            // Pause not at cue point and not at end position
+            cueSet(v);
+            // Just in case.
+            m_bPreviewing = false;
+            m_pPlay->set(0.0);
+            // If quantize is enabled, jump to the cue point since it's not
+            // necessarily where we currently are
+            if (m_pQuantizeEnabled->get() > 0.0) {
+                lock.unlock();  // prevent deadlock.
+                // Enginebuffer will quantize more exactly than we can.
+                seekAbs(m_pCuePoint->get());
+            }
+        }
+    } else if (isTrackAtCue()){
+        m_bPreviewing = false;
+        m_pPlay->set(1.0);
+        lock.unlock();
+
+    }
+}
+
 void CueControl::cueDefault(double v) {
     double cueMode = m_pCueMode->get();
     // Decide which cue implementation to call based on the user preference
     if (cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) {
         cueDenon(v);
+    } else if (cueMode == CUE_MODE_CUP) {
+        cuePlay(v);
     } else {
         // The modes CUE_MODE_PIONEER and CUE_MODE_MIXXX are similar
         // are handled inside cueCDJ(v)
@@ -815,7 +829,7 @@ void CueControl::pause(double v) {
     QMutexLocker lock(&m_mutex);
     //qDebug() << "CueControl::pause()" << v;
     if (v != 0.0) {
-        m_pPlayButton->set(0.0);
+        m_pPlay->set(0.0);
     }
 }
 
@@ -823,10 +837,10 @@ void CueControl::playStutter(double v) {
     QMutexLocker lock(&m_mutex);
     //qDebug() << "playStutter" << v;
     if (v != 0.0) {
-        if (m_pPlayButton->toBool()) {
+        if (isPlayingByPlayButton()) {
             cueGoto(1.0);
         } else {
-            m_pPlayButton->set(1.0);
+            m_pPlay->set(1.0);
         }
     }
 }
@@ -838,7 +852,7 @@ bool CueControl::updateIndicatorsAndModifyPlay(bool newPlay, bool playPossible)
     double cueMode = m_pCueMode->get();
     if ((cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) &&
         newPlay && playPossible &&
-        !m_pPlayButton->toBool() &&
+        !m_pPlay->toBool() &&
         !m_bypassCueSetByPlay) {
         // in Denon mode each play from pause moves the cue point
         // if not previewing
@@ -919,7 +933,7 @@ void CueControl::updateIndicators() {
 
     if (cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) {
         // Cue button is only lit at cue point
-        bool playing = m_pPlayButton->toBool();
+        bool playing = m_pPlay->toBool();
         if (isTrackAtCue()) {
             // at cue point
             if (!playing) {
@@ -942,8 +956,8 @@ void CueControl::updateIndicators() {
         // Here we have CUE_MODE_PIONEER or CUE_MODE_MIXXX
         // default to Pioneer mode
         if (!m_bPreviewing) {
-            bool playing = m_pPlayButton->toBool();
-            if (!playing) {
+            const auto freely_playing = m_pPlay->toBool() && !getEngineBuffer()->getScratching();
+            if (!freely_playing) {
                 if (!isTrackAtCue()) {
                     if (!atEndPosition()) {
                         if (cueMode == CUE_MODE_MIXXX) {
@@ -963,6 +977,9 @@ void CueControl::updateIndicators() {
                     // Next Press is preview
                     m_pCueIndicator->setBlinkValue(ControlIndicator::ON);
                 }
+            } else {
+                // Cue indicator should be off when freely playing
+                m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
             }
         }
     }
@@ -972,11 +989,17 @@ bool CueControl::isTrackAtCue() {
     return (fabs(getCurrentSample() - m_pCuePoint->get()) < 1.0f);
 }
 
-ConfigKey HotcueControl::keyForControl(int hotcue, QString name) {
+bool CueControl::isPlayingByPlayButton() {
+    return m_pPlay->toBool() &&
+            !m_iCurrentlyPreviewingHotcues && !m_bPreviewing;
+}
+
+
+ConfigKey HotcueControl::keyForControl(int hotcue, const char* name) {
     ConfigKey key;
     key.group = m_group;
     // Add one to hotcue so that we dont have a hotcue_0
-    key.item = QString("hotcue_%1_%2").arg(QString::number(hotcue+1), name);
+    key.item = QLatin1String("hotcue_") % QString::number(hotcue+1) % "_" % name;
     return key;
 }
 
@@ -993,7 +1016,7 @@ HotcueControl::HotcueControl(QString group, int i)
     m_hotcuePosition->set(-1);
 
     m_hotcueEnabled = new ControlObject(keyForControl(i, "enabled"));
-    m_hotcueEnabled->set(0);
+    m_hotcueEnabled->setReadOnly();
 
     m_hotcueSet = new ControlPushButton(keyForControl(i, "set"));
     connect(m_hotcueSet, SIGNAL(valueChanged(double)),
@@ -1072,5 +1095,29 @@ void HotcueControl::slotHotcueClear(double v) {
 }
 
 void HotcueControl::slotHotcuePositionChanged(double newPosition) {
+    m_hotcueEnabled->forceSet(newPosition == -1 ? 0.0 : 1.0);
     emit(hotcuePositionChanged(this, newPosition));
 }
+
+double HotcueControl::getPosition() const {
+    return m_hotcuePosition->get();
+}
+
+void HotcueControl::setCue(CuePointer pCue) {
+    setPosition(pCue->getPosition());
+    // set pCue only if all other data is in place
+    // because we have a null check for valid data else where in the code
+    m_pCue = pCue;
+}
+
+void HotcueControl::resetCue() {
+    // clear pCue first because we have a null check for valid data else where
+    // in the code
+    m_pCue.reset();
+    setPosition(-1.0);
+}
+
+void HotcueControl::setPosition(double position) {
+    m_hotcuePosition->set(position);
+    m_hotcueEnabled->forceSet(position == -1.0 ? 0.0 : 1.0);
+}
