@@ -1,71 +1,58 @@
-/***************************************************************************
-                          enginebuffer.cpp  -  description
-                             -------------------
-    begin                : Wed Feb 20 2002
-    copyright            : (C) 2002 by Tue and Ken Haste Andersen
-    email                :
-***************************************************************************/
-
-/***************************************************************************
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-***************************************************************************/
+#include "engine/enginebuffer.h"
 
 #include <QtDebug>
 
-#include "engine/enginebuffer.h"
-#include "cachingreader.h"
-#include "sampleutil.h"
-
-#include "controlpushbutton.h"
-#include "controlindicator.h"
-#include "configobject.h"
-#include "controlpotmeter.h"
-#include "controllinpotmeter.h"
-#include "engine/enginechannel.h"
-#include "engine/enginebufferscalest.h"
-#include "engine/enginebufferscalerubberband.h"
-#include "engine/enginebufferscalelinear.h"
-#include "engine/sync/enginesync.h"
-#include "engine/engineworkerscheduler.h"
-#include "engine/readaheadmanager.h"
-#include "engine/enginecontrol.h"
-#include "engine/loopingcontrol.h"
-#include "engine/ratecontrol.h"
+#include "engine/cachingreader.h"
+#include "preferences/usersettings.h"
+#include "control/controlindicator.h"
+#include "control/controllinpotmeter.h"
+#include "control/controlproxy.h"
+#include "control/controlpotmeter.h"
+#include "control/controlpushbutton.h"
 #include "engine/bpmcontrol.h"
-#include "engine/keycontrol.h"
-#include "engine/sync/synccontrol.h"
-#include "engine/quantizecontrol.h"
-#include "visualplayposition.h"
-#include "engine/cuecontrol.h"
 #include "engine/clockcontrol.h"
+#include "engine/cuecontrol.h"
+#include "engine/enginebufferscalelinear.h"
+#include "engine/enginebufferscalerubberband.h"
+#include "engine/enginebufferscalest.h"
+#include "engine/enginechannel.h"
+#include "engine/enginecontrol.h"
 #include "engine/enginemaster.h"
-#include "util/timer.h"
-#include "util/math.h"
-#include "util/defs.h"
+#include "engine/engineworkerscheduler.h"
+#include "engine/keycontrol.h"
+#include "engine/loopingcontrol.h"
+#include "engine/quantizecontrol.h"
+#include "engine/ratecontrol.h"
+#include "engine/readaheadmanager.h"
+#include "engine/sync/enginesync.h"
+#include "engine/sync/synccontrol.h"
 #include "track/beatfactory.h"
 #include "track/keyutils.h"
-#include "controlobjectslave.h"
-#include "util/compatibility.h"
+#include "track/track.h"
 #include "util/assert.h"
+#include "util/compatibility.h"
+#include "util/defs.h"
+#include "util/math.h"
+#include "util/sample.h"
+#include "util/timer.h"
+#include "waveform/visualplayposition.h"
 
 #ifdef __VINYLCONTROL__
 #include "engine/vinylcontrolcontrol.h"
 #endif
 
-#include "trackinfoobject.h"
+namespace {
 
 const double kLinearScalerElipsis = 1.00058; // 2^(0.01/12): changes < 1 cent allows a linear scaler
-const int kSamplesPerFrame = 2; // Engine buffer uses Stereo frames only
 
-EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
+const SINT kSamplesPerFrame = 2; // Engine buffer uses Stereo frames only
+
+} // anonymous namespace
+
+EngineBuffer::EngineBuffer(QString group, UserSettingsPointer pConfig,
                            EngineChannel* pChannel, EngineMaster* pMixingEngine)
         : m_group(group),
-          m_pConfig(_config),
+          m_pConfig(pConfig),
           m_pLoopingControl(NULL),
           m_pSyncControl(NULL),
           m_pVinylControlControl(NULL),
@@ -76,6 +63,7 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
           m_pReader(NULL),
           m_filepos_play(0.),
           m_speed_old(0),
+          m_tempo_ratio_old(1.),
           m_scratching_old(false),
           m_reverse_old(false),
           m_pitch_old(0),
@@ -97,33 +85,16 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
           m_iSeekPhaseQueued(0),
           m_iEnableSyncQueued(SYNC_REQUEST_NONE),
           m_iSyncModeQueued(SYNC_INVALID),
-          m_bLastBufferPaused(true),
           m_iTrackLoading(0),
           m_bPlayAfterLoading(false),
-          m_fRampValue(0.0),
-          m_iRampState(ENGINE_RAMP_NONE),
           m_iSampleRate(0),
-          m_pDitherBuffer(SampleUtil::alloc(MAX_BUFFER_LEN)),
-          m_iDitherBufferReadIndex(0),
           m_pCrossfadeBuffer(SampleUtil::alloc(MAX_BUFFER_LEN)),
           m_bCrossfadeReady(false),
           m_iLastBufferSize(0) {
-
-    // Generate dither values. When engine samples used to be within [SAMPLE_MIN,
-    // SAMPLE_MAX] dithering values were in the range [-0.5, 0.5]. Now that we
-    // normalize engine samples to the range [-1.0, 1.0] we divide by SAMPLE_MAX
-    // to preserve the previous behavior.
-    for (unsigned int i = 0; i < MAX_BUFFER_LEN; ++i) {
-        m_pDitherBuffer[i] = (static_cast<CSAMPLE>(rand() % RAND_MAX) / RAND_MAX - 0.5) / SAMPLE_MAX;
-    }
-
     // zero out crossfade buffer
     SampleUtil::clear(m_pCrossfadeBuffer, MAX_BUFFER_LEN);
 
-    m_fLastSampleValue[0] = 0;
-    m_fLastSampleValue[1] = 0;
-
-    m_pReader = new CachingReader(group, _config);
+    m_pReader = new CachingReader(group, pConfig);
     connect(m_pReader, SIGNAL(trackLoading()),
             this, SLOT(slotTrackLoading()),
             Qt::DirectConnection);
@@ -185,6 +156,9 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
     m_visualBpm = new ControlObject(ConfigKey(m_group, "visual_bpm"));
     m_visualKey = new ControlObject(ConfigKey(m_group, "visual_key"));
 
+    m_timeElapsed = new ControlObject(ConfigKey(m_group, "time_elapsed"));
+    m_timeRemaining = new ControlObject(ConfigKey(m_group, "time_remaining"));
+
     m_playposSlider = new ControlLinPotmeter(
         ConfigKey(m_group, "playposition"), 0.0, 1.0, 0, 0, true);
     connect(m_playposSlider, SIGNAL(valueChanged(double)),
@@ -198,9 +172,9 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
     m_pRepeat->setButtonMode(ControlPushButton::TOGGLE);
 
     // Sample rate
-    m_pSampleRate = new ControlObjectSlave("[Master]", "samplerate", this);
+    m_pSampleRate = new ControlProxy("[Master]", "samplerate", this);
 
-    m_pKeylockEngine = new ControlObjectSlave("[Master]", "keylock_engine", this);
+    m_pKeylockEngine = new ControlProxy("[Master]", "keylock_engine", this);
     m_pKeylockEngine->connectValueChanged(SLOT(slotKeylockEngineChanged(double)),
                                           Qt::DirectConnection);
 
@@ -215,13 +189,16 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
             this, SLOT(slotEjectTrack(double)),
             Qt::DirectConnection);
 
+    m_pTrackLoaded = new ControlObject(ConfigKey(m_group, "track_loaded"));
+    m_pTrackLoaded->setReadOnly();
+
     // Quantization Controller for enabling and disabling the
     // quantization (alignment) of loop in/out positions and (hot)cues with
     // beats.
-    QuantizeControl* quantize_control = new QuantizeControl(group, _config);
+    QuantizeControl* quantize_control = new QuantizeControl(group, pConfig);
 
     // Create the Loop Controller
-    m_pLoopingControl = new LoopingControl(group, _config);
+    m_pLoopingControl = new LoopingControl(group, pConfig);
     addControl(m_pLoopingControl);
 
     addControl(quantize_control);
@@ -229,20 +206,20 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
 
     m_pEngineSync = pMixingEngine->getEngineSync();
 
-    m_pSyncControl = new SyncControl(group, _config, pChannel, m_pEngineSync);
+    m_pSyncControl = new SyncControl(group, pConfig, pChannel, m_pEngineSync);
     addControl(m_pSyncControl);
 
 #ifdef __VINYLCONTROL__
-    m_pVinylControlControl = new VinylControlControl(group, _config);
+    m_pVinylControlControl = new VinylControlControl(group, pConfig);
     addControl(m_pVinylControlControl);
 #endif
 
-    m_pRateControl = new RateControl(group, _config);
+    m_pRateControl = new RateControl(group, pConfig);
     // Add the Rate Controller
     addControl(m_pRateControl);
 
     // Create the BPM Controller
-    m_pBpmControl = new BpmControl(group, _config);
+    m_pBpmControl = new BpmControl(group, pConfig);
     addControl(m_pBpmControl);
 
     // TODO(rryan) remove this dependence?
@@ -253,15 +230,15 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
     m_fwdButton = ControlObject::getControl(ConfigKey(group, "fwd"));
     m_backButton = ControlObject::getControl(ConfigKey(group, "back"));
 
-    m_pKeyControl = new KeyControl(group, _config);
+    m_pKeyControl = new KeyControl(group, pConfig);
     addControl(m_pKeyControl);
 
     // Create the clock controller
-    m_pClockControl = new ClockControl(group, _config);
+    m_pClockControl = new ClockControl(group, pConfig);
     addControl(m_pClockControl);
 
     // Create the cue controller
-    m_pCueControl = new CueControl(group, _config);
+    m_pCueControl = new CueControl(group, pConfig);
     addControl(m_pCueControl);
 
     m_pReadAheadManager = new ReadAheadManager(m_pReader,
@@ -282,11 +259,10 @@ EngineBuffer::EngineBuffer(QString group, ConfigObject<ConfigValue>* _config,
     m_pScale->clear();
     m_bScalerChanged = true;
 
-    m_pPassthroughEnabled = new ControlObjectSlave(group, "passthrough", this);
+    m_pPassthroughEnabled = new ControlProxy(group, "passthrough", this);
     m_pPassthroughEnabled->connectValueChanged(SLOT(slotPassthroughChanged(double)),
                                                Qt::DirectConnection);
 
-    //m_iRampIter = 0;
 #ifdef __SCALER_DEBUG__
     df.setFileName("mixxx-debug.csv");
     df.open(QIODevice::WriteOnly | QIODevice::Text);
@@ -312,6 +288,9 @@ EngineBuffer::~EngineBuffer() {
     delete m_playStartButton;
     delete m_stopStartButton;
 
+    delete m_timeElapsed;
+    delete m_timeRemaining;
+
     delete m_startButton;
     delete m_endButton;
     delete m_stopButton;
@@ -323,6 +302,7 @@ EngineBuffer::~EngineBuffer() {
     delete m_pRepeat;
     delete m_pSampleRate;
 
+    delete m_pTrackLoaded;
     delete m_pTrackSamples;
     delete m_pTrackSampleRate;
 
@@ -333,7 +313,6 @@ EngineBuffer::~EngineBuffer() {
     delete m_pKeylock;
     delete m_pEject;
 
-    SampleUtil::free(m_pDitherBuffer);
     SampleUtil::free(m_pCrossfadeBuffer);
 
     qDeleteAll(m_engineControls);
@@ -402,7 +381,7 @@ void EngineBuffer::queueNewPlaypos(double newpos, enum SeekRequest seekType) {
     // All seeks need to be done in the Engine thread so queue it up.
     // Write the position before the seek type, to reduce a possible race
     // condition effect
-    DEBUG_ASSERT_AND_HANDLE(seekType != SEEK_PHASE) {
+    VERIFY_OR_DEBUG_ASSERT(seekType != SEEK_PHASE) {
         // SEEK_PHASE with a position is not supported
         // use SEEK_STANDARD for that
         seekType = SEEK_STANDARD;
@@ -455,26 +434,23 @@ void EngineBuffer::readToCrossfadeBuffer(const int iBufferSize) {
     if (!m_bCrossfadeReady) {
         // Read buffer, as if there where no parameter change
         // (Must be called only once per callback)
-        CSAMPLE* fadeout = m_pScale->getScaled(iBufferSize);
-        SampleUtil::copy(m_pCrossfadeBuffer, fadeout, iBufferSize);
-
-        // Restore the original position that was lost due to getScaled() above
+        m_pScale->scaleBuffer(m_pCrossfadeBuffer, iBufferSize);
+        // Restore the original position that was lost due to scaleBuffer() above
         m_pReadAheadManager->notifySeek(m_filepos_play);
-
         m_bCrossfadeReady = true;
-    }
+     }
 }
 
 // WARNING: This method is not thread safe and must not be called from outside
 // the engine callback!
-void EngineBuffer::setNewPlaypos(double newpos) {
+void EngineBuffer::setNewPlaypos(double newpos, bool adjustingPhase) {
     //qDebug() << m_group << "engine new pos " << newpos;
 
     m_filepos_play = newpos;
 
     if (m_rate_old != 0.0) {
         // Before seeking, read extra buffer for crossfading
-        // (calls notifySeek())
+        // this also sets m_pReadAheadManager to newpos
         readToCrossfadeBuffer(m_iLastBufferSize);
     } else {
         m_pReadAheadManager->notifySeek(m_filepos_play);
@@ -488,22 +464,24 @@ void EngineBuffer::setNewPlaypos(double newpos) {
     for (QList<EngineControl*>::iterator it = m_engineControls.begin();
          it != m_engineControls.end(); ++it) {
         EngineControl *pControl = *it;
-        pControl->notifySeek(m_filepos_play);
+        pControl->notifySeek(m_filepos_play, adjustingPhase);
     }
 
     verifyPlay(); // verify or update play button and indicator
 }
 
-QString EngineBuffer::getGroup()
-{
+QString EngineBuffer::getGroup() {
     return m_group;
 }
 
-double EngineBuffer::getSpeed()
-{
+double EngineBuffer::getSpeed() {
     return m_speed_old;
 }
 
+bool EngineBuffer::getScratching() {
+    return m_scratching_old;
+}
+
 // WARNING: Always called from the EngineWorker thread pool
 void EngineBuffer::slotTrackLoading() {
     // Pause EngineBuffer from processing frames
@@ -518,20 +496,14 @@ void EngineBuffer::slotTrackLoading() {
     m_pTrackSamples->set(0); // Stop renderer
 }
 
-TrackPointer EngineBuffer::loadFakeTrack(double filebpm) {
-    TrackPointer pTrack(new TrackInfoObject(), &QObject::deleteLater);
-    pTrack->setSampleRate(44100);
-    // 10 seconds
-    pTrack->setDuration(10);
-    if (filebpm > 0) {
-        pTrack->setBpm(filebpm);
-        BeatsPointer pBeats = BeatFactory::makeBeatGrid(pTrack.data(), filebpm, 0.0);
-        pTrack->setBeats(pBeats);
+void EngineBuffer::loadFakeTrack(TrackPointer pTrack, bool bPlay) {
+    if (bPlay) {
+        m_playButton->set((double)bPlay);
     }
-    slotTrackLoaded(pTrack, 44100, 44100 * 10);
-    m_pSyncControl->setLocalBpm(filebpm);
-    m_pSyncControl->trackLoaded(pTrack);
-    return pTrack;
+    slotTrackLoaded(pTrack, pTrack->getSampleRate(),
+                    pTrack->getSampleRate() * pTrack->getDurationInt());
+    m_pSyncControl->setLocalBpm(pTrack->getBpm());
+    m_pSyncControl->trackLoaded(pTrack, TrackPointer());
 }
 
 // WARNING: Always called from the EngineWorker thread pool
@@ -539,6 +511,8 @@ void EngineBuffer::slotTrackLoaded(TrackPointer pTrack,
                                    int iTrackSampleRate,
                                    int iTrackNumSamples) {
     //qDebug() << getGroup() << "EngineBuffer::slotTrackLoaded";
+    TrackPointer pOldTrack = m_pCurrentTrack;
+
     m_pause.lock();
     m_visualPlayPos->setInvalid();
     m_pCurrentTrack = pTrack;
@@ -552,11 +526,12 @@ void EngineBuffer::slotTrackLoaded(TrackPointer pTrack,
     m_bSlipEnabledProcessing = false;
     m_dSlipPosition = 0.;
     m_dSlipRate = 0;
+    m_pTrackLoaded->forceSet(1);
     // Reset the pitch value for the new track.
     m_pause.unlock();
 
     // All EngineControls are connected directly
-    emit(trackLoaded(pTrack));
+    emit(trackLoaded(pTrack, pOldTrack));
     // Start buffer processing after all EngineContols are up to date
     // with the current track e.g track is seeked to Cue
     m_iTrackLoading = 0;
@@ -566,6 +541,8 @@ void EngineBuffer::slotTrackLoaded(TrackPointer pTrack,
 void EngineBuffer::slotTrackLoadFailed(TrackPointer pTrack,
                                        QString reason) {
     m_iTrackLoading = 0;
+    // Loading of a new track failed.
+    // eject the currently loaded track (the old Track) as well
     ejectTrack();
     emit(trackLoadFailed(pTrack, reason));
 }
@@ -576,22 +553,31 @@ TrackPointer EngineBuffer::getLoadedTrack() const {
 
 void EngineBuffer::ejectTrack() {
     // clear track values in any case, this may fix Bug #1450424
+    //qDebug() << "EngineBuffer::ejectTrack()";
     m_pause.lock();
     m_iTrackLoading = 0;
+    m_pTrackLoaded->forceSet(0);
     m_pTrackSamples->set(0);
     m_pTrackSampleRate->set(0);
     TrackPointer pTrack = m_pCurrentTrack;
-    m_pCurrentTrack.clear();
+    m_pCurrentTrack.reset();
     m_trackSampleRateOld = 0;
     m_trackSamplesOld = 0;
     m_playButton->set(0.0);
     m_visualBpm->set(0.0);
     m_visualKey->set(0.0);
+    m_timeElapsed->set(0);
+    m_timeRemaining->set(0);
+    m_playposSlider->set(0);
+    m_pCueControl->updateIndicators();
     doSeekFractional(0.0, SEEK_EXACT);
     m_pause.unlock();
 
+    // Close open file handles by unloading the current track
+    m_pReader->newTrack(TrackPointer());
+
     if (pTrack) {
-        emit(trackUnloaded(pTrack));
+        emit(trackLoaded(TrackPointer(), pTrack));
     }
 }
 
@@ -622,9 +608,8 @@ void EngineBuffer::doSeekFractional(double fractionalPos, enum SeekRequest seekT
     if (isnan(fractionalPos)) {
         return;
     }
-    // Find new play frame, restrict to valid ranges.
-    double newPlayFrame = round(fractionalPos * m_trackSamplesOld / kSamplesPerFrame);
-    doSeekPlayPos(newPlayFrame * kSamplesPerFrame, seekType);
+    double newSamplePosition = fractionalPos * m_trackSamplesOld;
+    doSeekPlayPos(newSamplePosition, seekType);
 }
 
 void EngineBuffer::doSeekPlayPos(double new_playpos, enum SeekRequest seekType) {
@@ -633,11 +618,6 @@ void EngineBuffer::doSeekPlayPos(double new_playpos, enum SeekRequest seekType)
         new_playpos = m_trackSamplesOld;
     }
 
-    // Ensure that the file position is even (remember, stereo channel files...)
-    if (!even(static_cast<int>(new_playpos))) {
-        new_playpos--;
-    }
-
 #ifdef __VINYLCONTROL__
     // Notify the vinyl control that a seek has taken place in case it is in
     // absolute mode and needs be switched to relative.
@@ -674,7 +654,19 @@ void EngineBuffer::verifyPlay() {
 }
 
 void EngineBuffer::slotControlPlayRequest(double v) {
+    bool oldPlay = m_playButton->toBool();
     bool verifiedPlay = updateIndicatorsAndModifyPlay(v > 0.0);
+
+    if (!oldPlay && verifiedPlay) {
+        if (m_pQuantize->get() > 0.0
+#ifdef __VINYLCONTROL__
+                && m_pVinylControlControl && !m_pVinylControlControl->isEnabled()
+#endif
+        ) {
+            requestSyncPhase();
+        }
+    }
+
     // set and confirm must be called here in any case to update the widget toggle state
     m_playButton->setAndConfirm(verifiedPlay ? 1.0 : 0.0);
 }
@@ -736,8 +728,8 @@ void EngineBuffer::slotKeylockEngineChanged(double dIndex) {
 }
 
 void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
-    // Bail if we receive a non-even buffer size. Assert in debug builds.
-    DEBUG_ASSERT_AND_HANDLE(even(iBufferSize)) {
+    // Bail if we receive a buffer size with incomplete sample frames. Assert in debug builds.
+    VERIFY_OR_DEBUG_ASSERT((iBufferSize % kSamplesPerFrame) == 0) {
         return;
     }
     m_pReader->process();
@@ -801,6 +793,11 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
         double speed = m_pRateControl->calculateSpeed(
                 baserate, tempoRatio, paused, iBufferSize, &is_scratching, &is_reverse);
 
+        // The cue indicator may change when scratch state is changed
+        if (is_scratching != m_scratching_old) {
+            m_pCueControl->updateIndicators();
+        }
+
         bool useIndependentPitchAndTempoScaling = false;
 
         // TODO(owen): Maybe change this so that rubberband doesn't disable
@@ -857,7 +854,7 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
             enableIndependentPitchTempoScaling(useIndependentPitchAndTempoScaling,
                                                iBufferSize);
         } else if (m_speed_old && !is_scratching) {
-            // we are stopped, collect samples for fade out
+            // we are stopping, collect samples for fade out
             readToCrossfadeBuffer(iBufferSize);
             // Clear the scaler information
             m_pScale->clear();
@@ -881,7 +878,7 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
         // the speedSliderPitchRatio is not reseted when keylock is enabled.
         // This mode allows to enable keylock
         // while the track is already played. You can reset to the tracks
-        // original pitch by reseting the pitch knob to center. When disabling
+        // original pitch by resetting the pitch knob to center. When disabling
         // keylock the pitch is reset to the linear vinyl pitch.
 
         // The Pitch knob turns if the speed slider is moved without keylock.
@@ -914,7 +911,8 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
         // scaler. Also, if we have changed scalers then we need to update the
         // scaler.
         if (baserate != m_baserate_old || speed != m_speed_old ||
-                pitchRatio != m_pitch_old || m_bScalerChanged) {
+                pitchRatio != m_pitch_old || tempoRatio != m_tempo_ratio_old ||
+                m_bScalerChanged) {
             // The rate returned by the scale object can be different from the
             // wanted rate!  Make sure new scaler has proper position. This also
             // crossfades between the old scaler and new scaler to prevent
@@ -937,6 +935,7 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
             m_baserate_old = baserate;
             m_speed_old = speed;
             m_pitch_old = pitchRatio;
+            m_tempo_ratio_old = tempoRatio;
             m_reverse_old = is_reverse;
 
             // Now we need to update the scaler with the master sample rate, the
@@ -970,10 +969,10 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
         if (at_end && !backwards) {
             // do not play past end
             bCurBufferPaused = true;
-        } else if (rate == 0 &&
-                (m_rate_old == 0 || !is_scratching)) {
+        } else if (rate == 0 && !is_scratching) {
             // do not process samples if have no transport
             // the linear scaler supports ramping down to 0
+            // this is used for pause by scratching only
             bCurBufferPaused = true;
         }
 
@@ -981,49 +980,44 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
 
         // If the buffer is not paused, then scale the audio.
         if (!bCurBufferPaused) {
-            //if (rate == 0) {
-            //    qDebug() << "ramp to rate 0";
-            //}
-
-            // The fileposition should be: (why is this thing a double anyway!?
-            // Integer valued.
-            double playFrame = m_filepos_play / kSamplesPerFrame;
-            double filepos_play_rounded = round(playFrame) * kSamplesPerFrame;
-            DEBUG_ASSERT_AND_HANDLE(filepos_play_rounded == m_filepos_play) {
-                qWarning() << __FILE__ << __LINE__ << "ERROR: filepos_play is not at an even integer sample:" << m_filepos_play;
-                m_filepos_play = filepos_play_rounded;
-            }
-
             // Perform scaling of Reader buffer into buffer.
-            CSAMPLE* output = m_pScale->getScaled(iBufferSize);
-            double samplesRead = m_pScale->getSamplesRead();
-
-            //qDebug() << "sourceSamples used " << iSourceSamples
-            //         <<" samplesRead " << samplesRead
-            //         << ", buffer pos " << iBufferStartSample
-            //         << ", play " << filepos_play
-            //         << " bufferlen " << iBufferSize;
-
-            // Copy scaled audio into pOutput
-            SampleUtil::copy(pOutput, output, iBufferSize);
+            double framesRead =
+                    m_pScale->scaleBuffer(pOutput, iBufferSize);
+            // TODO(XXX): The result framesRead might not be an integer value.
+            // Converting to samples here does not make sense. All positional
+            // calculations should be done in frames instead of samples! Otherwise
+            // rounding errors might occur when converting from samples back to
+            // frames later.
+            double samplesRead = framesRead * kSamplesPerFrame;
 
             if (m_bScalerOverride) {
                 // If testing, we don't have a real log so we fake the position.
                 m_filepos_play += samplesRead;
             } else {
-                // Adjust filepos_play by the amount we processed. TODO(XXX) what
-                // happens if samplesRead is a fraction ?
+                // Adjust filepos_play by the amount we processed.
                 m_filepos_play =
-                        m_pReadAheadManager->getEffectiveVirtualPlaypositionFromLog(
-                                static_cast<int>(m_filepos_play), samplesRead);
+                        m_pReadAheadManager->getFilePlaypositionFromLog(
+                                m_filepos_play, samplesRead);
             }
+            if (m_bCrossfadeReady) {
+                SampleUtil::linearCrossfadeBuffers(
+                        pOutput, m_pCrossfadeBuffer, pOutput, iBufferSize);
+            }
+            // Note: we do not fade here if we pass the end or the start of
+            // the track in reverse direction
+            // because we assume that the track samples itself start and stop
+            // towards zero.
+            // If it turns out that ramping is required be aware that the end
+            // or start may pass in the middle of the buffer.
         } else {
-            SampleUtil::clear(pOutput, iBufferSize);
-        }
-
-        if (m_bCrossfadeReady) {
-            SampleUtil::linearCrossfadeBuffers(
-                    pOutput, m_pCrossfadeBuffer, pOutput, iBufferSize);
+            // Pause
+            if (m_bCrossfadeReady) {
+                // We don't ramp here, since EnginePregain handles fades
+                // from and to speed == 0
+                SampleUtil::copy(pOutput, m_pCrossfadeBuffer, iBufferSize);
+            } else {
+                SampleUtil::clear(pOutput, iBufferSize);
+            }
         }
 
         QListIterator<EngineControl*> it(m_engineControls);
@@ -1055,83 +1049,35 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
             }
         }
 
-        // release the pauselock
-        m_pause.unlock();
-    } else { // if (!bTrackLoading && m_pause.tryLock()) {
-        // If we can't get the pause lock then this buffer will be silence.
-        bCurBufferPaused = true;
-
-        // We are stopped. Report a speed of 0 to SyncControl.
-        m_pSyncControl->reportPlayerSpeed(0.0, false);
-    }
-
-    if (!bTrackLoading) {
         // Give the Reader hints as to which chunks of the current song we
         // really care about. It will try very hard to keep these in memory
         hintReader(rate);
-    }
 
-    const double kSmallRate = 0.005;
-    if (m_bLastBufferPaused && !bCurBufferPaused) {
-        if (fabs(rate) > kSmallRate) { //at very slow forward rates, don't ramp up
-            m_iRampState = ENGINE_RAMP_UP;
-        }
-    } else if (!m_bLastBufferPaused && bCurBufferPaused) {
-        m_iRampState = ENGINE_RAMP_DOWN;
-    } else { //we are not changing state
-        // Make sure we aren't accidentally ramping down. This is how we make
-        // sure that ramp value will become 1.0 eventually.
-        //
-        // 9/2012 rryan -- As I understand it this code intends to prevent us
-        // from getting stuck ramped down. If there is a meaningfully large rate
-        // and we aren't ramped up completely then it makes us ramp up. This
-        // causes crazy feedback if you scratch at the non-silent end of a
-        // track. See Bug #1006111. I added a !bCurBufferPaused term here because
-        // if rate > 0 and bCurBufferPaused then basically you are at the end of
-        // the track and trying to jog forward so this uniquely blocks that
-        // situation.
-        if (fabs(rate) > kSmallRate && !bCurBufferPaused &&
-            m_iRampState != ENGINE_RAMP_UP && m_fRampValue < 1.0) {
-            m_iRampState = ENGINE_RAMP_UP;
-        }
-    }
+        // release the pauselock
+        m_pause.unlock();
+    } else { // if (!bTrackLoading && m_pause.tryLock()) {
+        // We are loading a new Track
+        bCurBufferPaused = true;
 
-    // let's try holding the last sample value constant, and pull it
-    // towards zero
-    float ramp_inc = 0;
-    if (m_iRampState == ENGINE_RAMP_UP ||
-        m_iRampState == ENGINE_RAMP_DOWN) {
-        // Ramp of 3.33 ms
-        ramp_inc = static_cast<double>(m_iRampState * 300) / sample_rate;
+        // Here the old track was playing and loading the new track is in
+        // progress. We can't predict when it happens, so we are not able
+        // to collect old samples. New samples are also not in place and
+        // we can't predict when they will be in place.
+        // If one does this, a click from breaking the last track is somehow
+        // natural and he should know that such sound should not be played to
+        // the master (audience).
+        // Workaround: Simply pause the track before.
 
-        for (int i=0; i < iBufferSize; i += 2) {
-            if (bCurBufferPaused && !m_bCrossfadeReady) {
-                //qDebug() << "ramp dither";
-                CSAMPLE dither = m_pDitherBuffer[m_iDitherBufferReadIndex];
-                m_iDitherBufferReadIndex = (m_iDitherBufferReadIndex + 1) % MAX_BUFFER_LEN;
-                pOutput[i] = m_fLastSampleValue[0] * m_fRampValue + dither;
-                pOutput[i+1] = m_fLastSampleValue[1] * m_fRampValue + dither;
-            } else {
-                //qDebug() << "ramp buffer";
-                pOutput[i] = pOutput[i] * m_fRampValue;
-                pOutput[i+1] = pOutput[i+1] * m_fRampValue;
-            }
+        // TODO(XXX):
+        // A click free solution requires more refactoring how loading a track
+        // is handled. For now we apply a rectangular Gain change here which
+        // may click.
 
-            m_fRampValue += ramp_inc;
-            if (m_fRampValue >= 1.0) {
-                m_iRampState = ENGINE_RAMP_NONE;
-                m_fRampValue = 1.0;
-            } else if (m_fRampValue <= 0.0) {
-                m_iRampState = ENGINE_RAMP_NONE;
-                m_fRampValue = 0.0;
-            }
-        }
-    }
+        SampleUtil::clear(pOutput, iBufferSize);
 
-    if ((!bCurBufferPaused && m_iRampState == ENGINE_RAMP_NONE) ||
-        (bCurBufferPaused && m_fRampValue == 0.0)) {
-        m_fLastSampleValue[0] = pOutput[iBufferSize-2];
-        m_fLastSampleValue[1] = pOutput[iBufferSize-1];
+        m_rate_old = 0;
+        m_speed_old = 0;
+        m_scratching_old = false;
     }
 
 #ifdef __SCALER_DEBUG__
@@ -1146,7 +1092,6 @@ void EngineBuffer::process(CSAMPLE* pOutput, const int iBufferSize) {
         m_pSyncControl->reportPlayerSpeed(m_speed_old, m_scratching_old);
     }
 
-    m_bLastBufferPaused = bCurBufferPaused;
     m_iLastBufferSize = iBufferSize;
     m_bCrossfadeReady = false;
 }
@@ -1203,7 +1148,7 @@ void EngineBuffer::processSyncRequests() {
 void EngineBuffer::processSeek(bool paused) {
     // We need to read position just after reading seekType, to ensure that we
     // read the matching position to seek_typ or a position from a new (second)
-    // seek just queued from an other thread
+    // seek just queued from another thread
     // The later case is ok, because we will process the new seek in the next
     // call anyway again.
 
@@ -1216,12 +1161,14 @@ void EngineBuffer::processSeek(bool paused) {
         seekType |= SEEK_PHASE;
     }
 
+    bool adjustingPhase = false;
     switch (seekType) {
         case SEEK_NONE:
             return;
         case SEEK_PHASE:
             // only adjust phase
             position = m_filepos_play;
+            adjustingPhase = true;
             break;
         case SEEK_EXACT:
         case SEEK_STANDARD: // = SEEK_EXACT | SEEK_PHASE
@@ -1233,13 +1180,10 @@ void EngineBuffer::processSeek(bool paused) {
     }
 
     if ((seekType & SEEK_PHASE) && !paused && m_pQuantize->toBool()) {
-        position += m_pBpmControl->getPhaseOffset(position);
+        position = m_pBpmControl->getNearestPositionInPhase(position, true, true);
     }
-
-    double newPlayFrame = position / kSamplesPerFrame;
-    position = round(newPlayFrame) * kSamplesPerFrame;
     if (position != m_filepos_play) {
-        setNewPlaypos(position);
+        setNewPlaypos(position, adjustingPhase);
     }
 }
 
@@ -1284,6 +1228,11 @@ void EngineBuffer::updateIndicators(double speed, int iBufferSize) {
     // Update indicators that are only updated after every
     // sampleRate/kiUpdateRate samples processed.  (e.g. playposSlider)
     if (m_iSamplesCalculated > (m_pSampleRate->get() / kiPlaypositionUpdateRate)) {
+        const double samplePositionToSeconds = 1.0 / m_trackSampleRateOld
+                / kSamplesPerFrame / m_tempo_ratio_old;
+        m_timeElapsed->set(m_filepos_play * samplePositionToSeconds);
+        m_timeRemaining->set(std::max(m_trackSamplesOld - m_filepos_play, 0.0) *
+                samplePositionToSeconds);
         m_playposSlider->set(fFractionalPlaypos);
         m_pCueControl->updateIndicators();
 
@@ -1312,37 +1261,41 @@ void EngineBuffer::hintReader(const double dRate) {
     //if slipping, hint about virtual position so we're ready for it
     if (m_bSlipEnabledProcessing) {
         Hint hint;
-        hint.length = 2048; //default length please
-        hint.sample = m_dSlipRate >= 0 ? m_dSlipPosition : m_dSlipPosition - 2048;
+        hint.frame = SampleUtil::floorPlayPosToFrame(m_dSlipPosition);
         hint.priority = 1;
+        if (m_dSlipRate >= 0) {
+            hint.frameCount = Hint::kFrameCountForward;
+        } else {
+            hint.frameCount = Hint::kFrameCountBackward;
+        }
         m_hintList.append(hint);
     }
 
-    QListIterator<EngineControl*> it(m_engineControls);
-    while (it.hasNext()) {
-        EngineControl* pControl = it.next();
+    for (const auto& pControl: m_engineControls) {
         pControl->hintReader(&m_hintList);
     }
     m_pReader->hintAndMaybeWake(m_hintList);
 }
 
 // WARNING: This method runs in the GUI thread
-void EngineBuffer::slotLoadTrack(TrackPointer pTrack, bool play) {
-    // Signal to the reader to load the track. The reader will respond with
-    // trackLoading and then either with trackLoaded or trackLoadFailed signals.
-    m_bPlayAfterLoading = play;
-    m_pReader->newTrack(pTrack);
+void EngineBuffer::loadTrack(TrackPointer pTrack, bool play) {
+    if (pTrack) {
+        // Signal to the reader to load the track. The reader will respond with
+        // trackLoading and then either with trackLoaded or trackLoadFailed signals.
+        m_bPlayAfterLoading = play;
+        m_pReader->newTrack(pTrack);
+    } else {
+        // Loading a null track means "eject"
+        ejectTrack();
+    }
 }
 
 void EngineBuffer::addControl(EngineControl* pControl) {
     // Connect to signals from EngineControl here...
     m_engineControls.push_back(pControl);
     pControl->setEngineBuffer(this);
-    connect(this, SIGNAL(trackLoaded(TrackPointer)),
-            pControl, SLOT(trackLoaded(TrackPointer)),
-            Qt::DirectConnection);
-    connect(this, SIGNAL(trackUnloaded(TrackPointer)),
-            pControl, SLOT(trackUnloaded(TrackPointer)),
+    connect(this, SIGNAL(trackLoaded(TrackPointer, TrackPointer)),
+            pControl, SLOT(trackLoaded(TrackPointer, TrackPointer)),
             Qt::DirectConnection);
 }
 
@@ -1406,13 +1359,7 @@ void EngineBuffer::setScalerForTest(EngineBufferScale* pScaleVinyl,
 }
 
 void EngineBuffer::collectFeatures(GroupFeatureState* pGroupFeatures) const {
-    pGroupFeatures->has_current_position = true;
-    pGroupFeatures->current_position = m_filepos_play;
-
     if (m_pBpmControl != NULL) {
         m_pBpmControl->collectFeatures(pGroupFeatures);
     }
-    if (m_pKeyControl != NULL) {
-        m_pKeyControl->collectFeatures(pGroupFeatures);
-    }
 }
