@@ -1,8 +1,8 @@
 #include "engine/sync/synccontrol.h"
 
-#include "controlobject.h"
-#include "controlpushbutton.h"
-#include "controlobjectslave.h"
+#include "control/controlobject.h"
+#include "control/controlpushbutton.h"
+#include "control/controlproxy.h"
 #include "engine/bpmcontrol.h"
 #include "engine/enginebuffer.h"
 #include "engine/enginechannel.h"
@@ -16,7 +16,7 @@ const double SyncControl::kBpmUnity = 1.0;
 const double SyncControl::kBpmHalve = 0.5;
 const double SyncControl::kBpmDouble = 2.0;
 
-SyncControl::SyncControl(const QString& group, ConfigObject<ConfigValue>* pConfig,
+SyncControl::SyncControl(const QString& group, UserSettingsPointer pConfig,
                          EngineChannel* pChannel, SyncableListener* pEngineSync)
         : EngineControl(group, pConfig),
           m_sGroup(group),
@@ -39,7 +39,7 @@ SyncControl::SyncControl(const QString& group, ConfigObject<ConfigValue>* pConfi
           m_pSyncPhaseButton(NULL) {
     // Play button.  We only listen to this to disable master if the deck is
     // stopped.
-    m_pPlayButton = new ControlObjectSlave(group, "play", this);
+    m_pPlayButton = new ControlProxy(group, "play", this);
     m_pPlayButton->connectValueChanged(SLOT(slotControlPlay(double)),
                                        Qt::DirectConnection);
 
@@ -64,11 +64,11 @@ SyncControl::SyncControl(const QString& group, ConfigObject<ConfigValue>* pConfi
     m_pSyncBeatDistance.reset(
             new ControlObject(ConfigKey(group, "beat_distance")));
 
-    m_pPassthroughEnabled = new ControlObjectSlave(group, "passthrough", this);
+    m_pPassthroughEnabled = new ControlProxy(group, "passthrough", this);
     m_pPassthroughEnabled->connectValueChanged(
             SLOT(slotPassthroughChanged(double)), Qt::DirectConnection);
 
-    m_pEjectButton = new ControlObjectSlave(group, "eject", this);
+    m_pEjectButton = new ControlProxy(group, "eject", this);
     m_pEjectButton->connectValueChanged(
             SLOT(slotEjectPushed(double)), Qt::DirectConnection);
 
@@ -86,30 +86,30 @@ void SyncControl::setEngineControls(RateControl* pRateControl,
     // We set this to change the effective BPM in BpmControl. We do not listen
     // to changes from this control because changes in rate, rate_dir, rateRange
     // and file_bpm result in changes to this control.
-    m_pBpm = new ControlObjectSlave(getGroup(), "bpm", this);
+    m_pBpm = new ControlProxy(getGroup(), "bpm", this);
 
-    m_pLocalBpm = new ControlObjectSlave(getGroup(), "local_bpm", this);
+    m_pLocalBpm = new ControlProxy(getGroup(), "local_bpm", this);
 
-    m_pFileBpm = new ControlObjectSlave(getGroup(), "file_bpm", this);
+    m_pFileBpm = new ControlProxy(getGroup(), "file_bpm", this);
     m_pFileBpm->connectValueChanged(SLOT(slotFileBpmChanged()),
                                     Qt::DirectConnection);
 
-    m_pRateSlider = new ControlObjectSlave(getGroup(), "rate", this);
+    m_pRateSlider = new ControlProxy(getGroup(), "rate", this);
     m_pRateSlider->connectValueChanged(SLOT(slotRateChanged()),
                                        Qt::DirectConnection);
 
-    m_pRateDirection = new ControlObjectSlave(getGroup(), "rate_dir", this);
+    m_pRateDirection = new ControlProxy(getGroup(), "rate_dir", this);
     m_pRateDirection->connectValueChanged(SLOT(slotRateChanged()),
                                           Qt::DirectConnection);
 
-    m_pRateRange = new ControlObjectSlave(getGroup(), "rateRange", this);
+    m_pRateRange = new ControlProxy(getGroup(), "rateRange", this);
     m_pRateRange->connectValueChanged(SLOT(slotRateChanged()),
                                       Qt::DirectConnection);
 
-    m_pSyncPhaseButton = new ControlObjectSlave(getGroup(), "beatsync_phase", this);
+    m_pSyncPhaseButton = new ControlProxy(getGroup(), "beatsync_phase", this);
 
 #ifdef __VINYLCONTROL__
-    m_pVCEnabled = new ControlObjectSlave(
+    m_pVCEnabled = new ControlProxy(
             getGroup(), "vinylcontrol_enabled", this);
 
     // Throw a hissy fit if somebody moved us such that the vinylcontrol_enabled
@@ -147,11 +147,11 @@ void SyncControl::notifySyncModeChanged(SyncMode mode) {
         m_pPassthroughEnabled->set(0.0);
     }
     if (mode == SYNC_MASTER) {
-        // Make sure all the slaves update based on our current rate.
+        // Make sure all the followers update based on our current rate.
         slotRateChanged();
-        double dRate = 1.0 + m_pRateDirection->get() * m_pRateRange->get() * m_pRateSlider->get();
+        double rateRatio = calcRateRatio();
         m_pEngineSync->notifyBeatDistanceChanged(this, getBeatDistance());
-        m_pBpm->set(m_pLocalBpm->get() * dRate);
+        m_pBpm->set(m_pLocalBpm->get() * rateRatio);
     }
 }
 
@@ -231,9 +231,11 @@ void SyncControl::setMasterBpm(double bpm) {
     }
 
     double localBpm = m_pLocalBpm->get();
-    if (localBpm > 0.0) {
-        double newRate = (bpm * m_masterBpmAdjustFactor / m_pLocalBpm->get() - 1.0)
-                / m_pRateDirection->get() / m_pRateRange->get();
+    double rateRange = m_pRateRange->get();
+    if (localBpm > 0.0 && rateRange > 0.0) {
+        double newRate = m_pRateDirection->get() *
+                ((bpm * m_masterBpmAdjustFactor / localBpm) - 1.0) /
+                rateRange;
         m_pRateSlider->set(newRate);
     } else {
         m_pRateSlider->set(0);
@@ -249,6 +251,9 @@ void SyncControl::setMasterParams(double beatDistance, double baseBpm, double bp
 
 double SyncControl::determineBpmMultiplier(double myBpm, double targetBpm) const {
     double multiplier = kBpmUnity;
+    if (myBpm == 0.0) {
+        return multiplier;
+    }
     double best_margin = fabs((targetBpm / myBpm) - 1.0);
 
     double try_margin = fabs((targetBpm * kBpmHalve / myBpm) - 1.0);
@@ -306,34 +311,27 @@ void SyncControl::reportTrackPosition(double fractionalPlaypos) {
     }
 }
 
-void SyncControl::trackLoaded(TrackPointer pTrack) {
+void SyncControl::trackLoaded(TrackPointer pNewTrack, TrackPointer pOldTrack) {
+    Q_UNUSED(pOldTrack);
     //qDebug() << getGroup() << "SyncControl::trackLoaded";
-    Q_UNUSED(pTrack);
-    m_masterBpmAdjustFactor = kBpmUnity;
     if (getSyncMode() == SYNC_MASTER) {
-        // If we loaded a new track while master, hand off.
+        // If we change or remove a new track while master, hand off.
         m_pChannel->getEngineBuffer()->requestSyncMode(SYNC_NONE);
     }
-
-    if (getSyncMode() != SYNC_NONE) {
-        // Because of the order signals get processed, the file/local_bpm COs and
-        // rate slider are not updated as soon as we need them, so do that now.
-        m_pFileBpm->set(pTrack->getBpm());
-        m_pLocalBpm->set(pTrack->getBpm());
-        double dRate = 1.0 + m_pRateDirection->get() * m_pRateRange->get() * m_pRateSlider->get();
-        // We used to set the m_pBpm here, but that causes a signal loop whereby
-        // that was interpretted as a rate slider tweak, and the master bpm
-        // was changed.  Instead, now we pass the suggested bpm to enginesync
-        // explicitly, and it can decide what to do with it.
-        m_pEngineSync->notifyTrackLoaded(this, m_pLocalBpm->get() * dRate);
-    }
-}
-
-void SyncControl::trackUnloaded(TrackPointer pTrack) {
-    Q_UNUSED(pTrack);
-    if (getSyncMode() == SYNC_MASTER) {
-        // If we unloaded a new track while master, hand off.
-        m_pChannel->getEngineBuffer()->requestSyncMode(SYNC_NONE);
+    if (pNewTrack) {
+        m_masterBpmAdjustFactor = kBpmUnity;
+        if (getSyncMode() != SYNC_NONE) {
+            // Because of the order signals get processed, the file/local_bpm COs and
+            // rate slider are not updated as soon as we need them, so do that now.
+            m_pFileBpm->set(pNewTrack->getBpm());
+            m_pLocalBpm->set(pNewTrack->getBpm());
+            double dRate = calcRateRatio();
+            // We used to set the m_pBpm here, but that causes a signal loop whereby
+            // that was interpretted as a rate slider tweak, and the master bpm
+            // was changed.  Instead, now we pass the suggested bpm to enginesync
+            // explicitly, and it can decide what to do with it.
+            m_pEngineSync->notifyTrackLoaded(this, m_pLocalBpm->get() * dRate);
+        }
     }
 }
 
@@ -424,8 +422,8 @@ void SyncControl::setLocalBpm(double local_bpm) {
     m_prevLocalBpm = local_bpm;
 
     // FIXME: This recalculating of the rate is duplicated in bpmcontrol.
-    const double rate = 1.0 + m_pRateSlider->get() * m_pRateRange->get() * m_pRateDirection->get();
-    double bpm = local_bpm * rate;
+    const double rateRatio = calcRateRatio();
+    double bpm = local_bpm * rateRatio;
     m_pBpm->set(bpm);
     m_pEngineSync->notifyBpmChanged(this, bpm, true);
 }
@@ -438,8 +436,8 @@ void SyncControl::slotFileBpmChanged() {
 
 void SyncControl::slotRateChanged() {
     // This slot is fired by rate, rate_dir, and rateRange changes.
-    const double rate = 1.0 + m_pRateSlider->get() * m_pRateRange->get() * m_pRateDirection->get();
-    double bpm = m_pLocalBpm ? m_pLocalBpm->get() * rate : 0.0;
+    const double rateRatio = calcRateRatio();
+    double bpm = m_pLocalBpm ? m_pLocalBpm->get() * rateRatio : 0.0;
     //qDebug() << getGroup() << "SyncControl::slotRateChanged" << rate << bpm;
     if (bpm > 0) {
         // When reporting our bpm, remove the multiplier so the masters all
@@ -460,3 +458,9 @@ void SyncControl::reportPlayerSpeed(double speed, bool scratching) {
     double instantaneous_bpm = m_pLocalBpm->get() * speed / m_masterBpmAdjustFactor;
     m_pEngineSync->notifyInstantaneousBpmChanged(this, instantaneous_bpm);
 }
+
+double SyncControl::calcRateRatio() {
+    double rateRatio = 1.0 + m_pRateDirection->get() * m_pRateRange->get() *
+            m_pRateSlider->get();
+    return rateRatio;
+}
