@@ -8,17 +8,15 @@
 #include "library/browse/browsetablemodel.h"
 #include "library/browse/browsethread.h"
 #include "library/previewbuttondelegate.h"
-#include "soundsourceproxy.h"
-#include "playerinfo.h"
-#include "controlobject.h"
+#include "mixer/playerinfo.h"
+#include "control/controlobject.h"
 #include "library/dao/trackdao.h"
-#include "audiotagger.h"
 #include "util/dnd.h"
 
 BrowseTableModel::BrowseTableModel(QObject* parent,
                                    TrackCollection* pTrackCollection,
                                    RecordingManager* pRecordingManager)
-        : TrackModel(pTrackCollection->getDatabase(),
+        : TrackModel(pTrackCollection->database(),
                      "mixxx.db.model.browse"),
           QStandardItemModel(parent),
           m_pTrackCollection(pTrackCollection),
@@ -40,7 +38,8 @@ BrowseTableModel::BrowseTableModel(QObject* parent,
     header_data.insert(COLUMN_KEY, tr("Key"));
     header_data.insert(COLUMN_TYPE, tr("Type"));
     header_data.insert(COLUMN_BITRATE, tr("Bitrate"));
-    header_data.insert(COLUMN_LOCATION, tr("Location"));
+    header_data.insert(COLUMN_REPLAYGAIN, tr("ReplayGain"));
+    header_data.insert(COLUMN_NATIVELOCATION, tr("Location"));
     header_data.insert(COLUMN_ALBUMARTIST, tr("Album Artist"));
     header_data.insert(COLUMN_GROUPING, tr("Grouping"));
     header_data.insert(COLUMN_FILE_MODIFIED_TIME, tr("File Modified"));
@@ -109,6 +108,14 @@ TrackPointer BrowseTableModel::getTrack(const QModelIndex& index) const {
             + "\n" +track_location);
         return TrackPointer();
     }
+    // NOTE(uklotzde, 2015-12-08): Accessing tracks from the browse view
+    // will implicitly add them to the library. Is this really what we
+    // want here??
+    // NOTE(rryan, 2015-12-27): This was intentional at the time since
+    // some people use Browse instead of the library and we want to let
+    // them edit the tracks in a way that persists across sessions
+    // and we didn't want to edit the files on disk by default
+    // unless the user opts in to that.
     return m_pTrackCollection->getTrackDAO()
             .getOrAddTrack(track_location, true, NULL);
 }
@@ -116,17 +123,25 @@ TrackPointer BrowseTableModel::getTrack(const QModelIndex& index) const {
 QString BrowseTableModel::getTrackLocation(const QModelIndex& index) const {
     int row = index.row();
 
-    QModelIndex index2 = this->index(row, COLUMN_LOCATION);
-    return data(index2).toString();
+    QModelIndex index2 = this->index(row, COLUMN_NATIVELOCATION);
+    QString nativeLocation = data(index2).toString();
+    QString location = QDir::fromNativeSeparators(nativeLocation);
+    return location;
 }
 
-int BrowseTableModel::getTrackId(const QModelIndex& index) const {
-    Q_UNUSED(index);
-    // We can't implement this as it stands.
-    return -1;
+TrackId BrowseTableModel::getTrackId(const QModelIndex& index) const {
+    TrackPointer pTrack = getTrack(index);
+    if (pTrack) {
+        return pTrack->getId();
+    } else {
+        qWarning()
+                << "Track is not available in library"
+                << getTrackLocation(index);
+        return TrackId();
+    }
 }
 
-const QLinkedList<int> BrowseTableModel::getTrackRows(int trackId) const {
+const QLinkedList<int> BrowseTableModel::getTrackRows(TrackId trackId) const {
     Q_UNUSED(trackId);
     // We can't implement this as it stands.
     return QLinkedList<int>();
@@ -150,9 +165,10 @@ bool BrowseTableModel::isColumnHiddenByDefault(int column) {
             column == COLUMN_TRACK_NUMBER ||
             column == COLUMN_YEAR ||
             column == COLUMN_GROUPING ||
-            column == COLUMN_LOCATION ||
+            column == COLUMN_NATIVELOCATION ||
             column == COLUMN_ALBUMARTIST ||
-            column == COLUMN_FILE_CREATION_TIME) {
+            column == COLUMN_FILE_CREATION_TIME ||
+            column == COLUMN_REPLAYGAIN) {
         return true;
     }
     return false;
@@ -161,77 +177,7 @@ bool BrowseTableModel::isColumnHiddenByDefault(int column) {
 void BrowseTableModel::moveTrack(const QModelIndex&, const QModelIndex&) {
 }
 
-void BrowseTableModel::removeTrack(const QModelIndex& index) {
-    if (!index.isValid()) {
-        return;
-    }
-    QStringList trackLocations;
-    trackLocations.append(getTrackLocation(index));
-    removeTracks(trackLocations);
-}
-
-void BrowseTableModel::removeTracks(const QModelIndexList& indices) {
-    QStringList trackLocations;
-    foreach (QModelIndex index, indices) {
-        if (!index.isValid()) {
-            continue;
-        }
-        trackLocations.append(getTrackLocation(index));
-    }
-    removeTracks(trackLocations);
-}
-
-void BrowseTableModel::removeTracks(QStringList trackLocations) {
-    if (trackLocations.size() == 0)
-        return;
-
-    // Ask user if s/he is sure
-    if (QMessageBox::question(
-        NULL, tr("Mixxx Library"),
-        tr("Warning: This will permanently delete the following files:")
-        + "\n" + trackLocations.join("\n") + "\n" +
-        tr("Are you sure you want to delete these files from your computer?"),
-        QMessageBox::Yes, QMessageBox::Abort) == QMessageBox::Abort) {
-        return;
-    }
-
-    QList<int> deleted_ids;
-    bool any_deleted = false;
-    TrackDAO& track_dao = m_pTrackCollection->getTrackDAO();
-
-    foreach (QString track_location, trackLocations) {
-        // If track is in use or deletion fails, show an error message.
-        if (isTrackInUse(track_location) || !QFile::remove(track_location)) {
-            QMessageBox::critical(
-                0, tr("Mixxx Library"),
-                tr("Could not delete the following file because"
-                   " it is in use by Mixxx or another application:") + "\n" +track_location);
-            continue;
-        }
-
-        qDebug() << "BrowseFeature: User deleted track " << track_location;
-        any_deleted = true;
-
-        deleted_ids.append(track_dao.getTrackId(track_location));
-    }
-
-    // If the tracks are contained in the Mixxx library, delete them
-    if (!deleted_ids.isEmpty()) {
-        qDebug() << "BrowseFeature: Purge affected track from database";
-        track_dao.purgeTracks(deleted_ids);
-    }
-
-    // Repopulate model if any tracks were actually deleted
-    if (any_deleted) {
-        m_pBrowseThread->executePopulation(m_current_directory,
-                                                       this);
-    }
-}
-
-bool BrowseTableModel::addTrack(const QModelIndex& index, QString location) {
-    Q_UNUSED(index);
-    Q_UNUSED(location);
-    return false;
+void BrowseTableModel::removeTracks(const QModelIndexList&) {
 }
 
 QMimeData* BrowseTableModel::mimeData(const QModelIndexList &indexes) const {
@@ -300,105 +246,93 @@ Qt::ItemFlags BrowseTableModel::flags(const QModelIndex &index) const {
     QString track_location = getTrackLocation(index);
     int column = index.column();
 
-    if (isTrackInUse(track_location) ||
-            column == COLUMN_FILENAME ||
-            column == COLUMN_BITRATE ||
-            column == COLUMN_DURATION ||
-            column == COLUMN_TYPE ||
-            column == COLUMN_FILE_MODIFIED_TIME ||
-            column == COLUMN_FILE_CREATION_TIME) {
+    switch (column) {
+    case COLUMN_FILENAME:
+    case COLUMN_BITRATE:
+    case COLUMN_DURATION:
+    case COLUMN_TYPE:
+    case COLUMN_FILE_MODIFIED_TIME:
+    case COLUMN_FILE_CREATION_TIME:
+    case COLUMN_REPLAYGAIN:
+        // read-only
         return defaultFlags;
-    } else {
+    default:
+        // editable
         return defaultFlags | Qt::ItemIsEditable;
     }
 }
 
-bool BrowseTableModel::isTrackInUse(const QString& track_location) const {
-    if (PlayerInfo::instance().isFileLoaded(track_location)) {
-        return true;
-    }
-
-    if (m_pRecordingManager->getRecordingLocation() == track_location) {
-        return true;
-    }
-
-    return false;
-}
-
-bool BrowseTableModel::setData(const QModelIndex &index, const QVariant &value,
-                               int role) {
+bool BrowseTableModel::setData(
+        const QModelIndex& index,
+        const QVariant& value,
+        int role) {
     Q_UNUSED(role);
 
-    if (!index.isValid()) {
-        return false;
-    }
-    qDebug() << "BrowseTableModel::setData(" << index.data() << ")";
-    int row = index.row();
-    int col = index.column();
-    QString track_location = getTrackLocation(index);
-    AudioTagger tagger(track_location, m_current_directory.token());
-
-    // set tagger information
-    tagger.setArtist(this->index(row, COLUMN_ARTIST).data().toString());
-    tagger.setTitle(this->index(row, COLUMN_TITLE).data().toString());
-    tagger.setAlbum(this->index(row, COLUMN_ALBUM).data().toString());
-    tagger.setKey(this->index(row, COLUMN_KEY).data().toString());
-    tagger.setBpm(this->index(row, COLUMN_BPM).data().toString());
-    tagger.setComment(this->index(row, COLUMN_COMMENT).data().toString());
-    tagger.setTracknumber(
-        this->index(row, COLUMN_TRACK_NUMBER).data().toString());
-    tagger.setYear(this->index(row, COLUMN_YEAR).data().toString());
-    tagger.setGenre(this->index(row, COLUMN_GENRE).data().toString());
-    tagger.setComposer(this->index(row, COLUMN_COMPOSER).data().toString());
-    tagger.setAlbumArtist(this->index(row, COLUMN_ALBUMARTIST).data().toString());
-    tagger.setGrouping(this->index(row, COLUMN_GROUPING).data().toString());
-
-    // check if one the item were edited
-    if (col == COLUMN_ARTIST) {
-        tagger.setArtist(value.toString());
-    } else if (col == COLUMN_TITLE) {
-        tagger.setTitle(value.toString());
-    } else if (col == COLUMN_ALBUM) {
-        tagger.setAlbum(value.toString());
-    } else if (col == COLUMN_BPM) {
-        tagger.setBpm(value.toString());
-    } else if (col == COLUMN_KEY) {
-        tagger.setKey(value.toString());
-    } else if (col == COLUMN_TRACK_NUMBER) {
-        tagger.setTracknumber(value.toString());
-    } else if (col == COLUMN_COMMENT) {
-        tagger.setComment(value.toString());
-    } else if (col == COLUMN_GENRE) {
-        tagger.setGenre(value.toString());
-    } else if (col == COLUMN_COMPOSER) {
-        tagger.setComposer(value.toString());
-    } else if (col == COLUMN_YEAR) {
-        tagger.setYear(value.toString());
-    } else if (col == COLUMN_ALBUMARTIST) {
-        tagger.setAlbumArtist(value.toString());
-    } else if (col == COLUMN_GROUPING) {
-        tagger.setGrouping(value.toString());
-    } else {
-        qWarning() << "BrowseTableModel::setData(): no tagger column";
-        return false;
-    }
-
     QStandardItem* item = itemFromIndex(index);
-    if (tagger.save()) {
-        // Modify underlying interalPointer object
-        item->setText(value.toString());
-        item->setToolTip(item->text());
-        return true;
-    } else {
-        // reset to old value in error
+    DEBUG_ASSERT(nullptr != item);
+
+    TrackPointer pTrack(getTrack(index));
+    if (!pTrack) {
+        qWarning() << "BrowseTableModel::setData():"
+                << "Failed to resolve track"
+                << getTrackLocation(index);
+        // restore previous item content
         item->setText(index.data().toString());
         item->setToolTip(item->text());
-        QMessageBox::critical(
-            0, tr("Mixxx Library"),
-            tr("Could not update file metadata.")
-            + "\n" +track_location);
         return false;
     }
+
+    // check if one the item were edited
+    int col = index.column();
+    switch (col) {
+    case COLUMN_ARTIST:
+        pTrack->setArtist(value.toString());
+        break;
+    case COLUMN_TITLE:
+        pTrack->setTitle(value.toString());
+        break;
+    case COLUMN_ALBUM:
+        pTrack->setAlbum(value.toString());
+        break;
+    case COLUMN_BPM:
+        pTrack->setBpm(value.toDouble());
+        break;
+    case COLUMN_KEY:
+        pTrack->setKeyText(value.toString());
+        break;
+    case COLUMN_TRACK_NUMBER:
+        pTrack->setTrackNumber(value.toString());
+        break;
+    case COLUMN_COMMENT:
+        pTrack->setComment(value.toString());
+        break;
+    case COLUMN_GENRE:
+        pTrack->setGenre(value.toString());
+        break;
+    case COLUMN_COMPOSER:
+        pTrack->setComposer(value.toString());
+        break;
+    case COLUMN_YEAR:
+        pTrack->setYear(value.toString());
+        break;
+    case COLUMN_ALBUMARTIST:
+        pTrack->setAlbumArtist(value.toString());
+        break;
+    case COLUMN_GROUPING:
+        pTrack->setGrouping(value.toString());
+        break;
+    default:
+        qWarning() << "BrowseTableModel::setData():"
+            << "No tagger column";
+        // restore previous item context
+        item->setText(index.data().toString());
+        item->setToolTip(item->text());
+        return false;
+    }
+
+    item->setText(value.toString());
+    item->setToolTip(item->text());
+    return true;
 }
 
 void BrowseTableModel::trackLoaded(QString group, TrackPointer pTrack) {
@@ -411,10 +345,11 @@ void BrowseTableModel::trackLoaded(QString group, TrackPointer pTrack) {
             }
         }
         if (pTrack) {
+            QString trackLocation = pTrack->getLocation();
             for (int row = 0; row < rowCount(); ++row) {
                 QModelIndex i = index(row, COLUMN_PREVIEW);
-                QString location = index(row, COLUMN_LOCATION).data().toString();
-                if (location == pTrack->getLocation()) {
+                QString location = getTrackLocation(i);
+                if (location == trackLocation) {
                     QStandardItem* item = itemFromIndex(i);
                     item->setText("1");
                     break;
@@ -425,10 +360,7 @@ void BrowseTableModel::trackLoaded(QString group, TrackPointer pTrack) {
 }
 
 bool BrowseTableModel::isColumnSortable(int column) {
-    if (COLUMN_PREVIEW == column) {
-        return false;
-    }
-    return true;
+    return COLUMN_PREVIEW != column;
 }
 
 QAbstractItemDelegate* BrowseTableModel::delegateForColumn(const int i, QObject* pParent) {
