@@ -68,7 +68,7 @@ ITunesFeature::ITunesFeature(QObject* parent, TrackCollection* pTrackCollection)
     m_isActivated = false;
     m_title = tr("iTunes");
 
-    m_database = QSqlDatabase::cloneDatabase(pTrackCollection->getDatabase(), "ITUNES_SCANNER");
+    m_database = QSqlDatabase::cloneDatabase(pTrackCollection->database(), "ITUNES_SCANNER");
 
     //Open the database connection in this thread.
     if (!m_database.open()) {
@@ -120,7 +120,7 @@ void ITunesFeature::activate() {
 void ITunesFeature::activate(bool forceReload) {
     //qDebug("ITunesFeature::activate()");
     if (!m_isActivated || forceReload) {
-        SettingsDAO settings(m_pTrackCollection->getDatabase());
+        SettingsDAO settings(m_pTrackCollection->database());
         QString dbSetting(settings.getValue(ITDB_PATH_KEY));
         // if a path exists in the database, use it
         if (!dbSetting.isEmpty() && QFile::exists(dbSetting)) {
@@ -324,6 +324,9 @@ void ITunesFeature::guessMusicLibraryMountpoint(QXmlStreamReader &xml) {
 // This method is executed in a separate thread
 // via QtConcurrent::run
 TreeItem* ITunesFeature::importLibrary() {
+    bool isTracksParsed=false;
+    bool isMusicFolderLocatedAfterTracks=false;
+  
     //Give thread a low priority
     QThread* thisThread = QThread::currentThread();
     thisThread->setPriority(QThread::LowPriority);
@@ -362,6 +365,7 @@ TreeItem* ITunesFeature::importLibrary() {
             if (xml.name() == "key") {
                 QString key = xml.readElementText();
                 if (key == "Music Folder") {
+                    if (isTracksParsed) isMusicFolderLocatedAfterTracks = true;
                     if (readNextStartElement(xml)) {
                         guessMusicLibraryMountpoint(xml);
                     }
@@ -370,14 +374,29 @@ TreeItem* ITunesFeature::importLibrary() {
                     if (playlist_root != NULL)
                         delete playlist_root;
                     playlist_root = parsePlaylists(xml);
+                    isTracksParsed = true;
                 }
             }
         }
     }
 
     itunes_file.close();
+    
+    if (isMusicFolderLocatedAfterTracks) {
+        qDebug() << "Updating iTunes real path from " << m_dbItunesRoot << " to " << m_mixxxItunesRoot;
+        // In some iTunes files "Music Folder" XML node is located at the end of file. So, we need to 
+        QSqlQuery query(m_database);
+        query.prepare("UPDATE itunes_library SET location = replace( location, :itunes_path, :mixxx_path )");
+        query.bindValue(":itunes_path", m_dbItunesRoot.replace(localhost_token(), ""));
+        query.bindValue(":mixxx_path", m_mixxxItunesRoot);
+        bool success = query.exec();
 
-    // Even if an error occured, commit the transaction. The file may have been
+        if (!success) {
+            LOG_FAILED_QUERY(query);
+        }
+    }
+
+    // Even if an error occurred, commit the transaction. The file may have been
     // half-parsed.
     transaction.commit();
 
@@ -587,7 +606,7 @@ void ITunesFeature::parseTrack(QXmlStreamReader &xml, QSqlQuery &query) {
 
 TreeItem* ITunesFeature::parsePlaylists(QXmlStreamReader &xml) {
     qDebug() << "Parse iTunes playlists";
-    TreeItem* rootItem = new TreeItem();
+    TreeItem* rootItem = new TreeItem(this);
     QSqlQuery query_insert_to_playlists(m_database);
     query_insert_to_playlists.prepare("INSERT INTO itunes_playlists (id, name) "
                                       "VALUES (:id, :name)");
@@ -636,6 +655,7 @@ void ITunesFeature::parsePlaylist(QXmlStreamReader &xml, QSqlQuery &query_insert
     int track_reference = -1;
     //indicates that we haven't found the <
     bool isSystemPlaylist = false;
+    bool isPlaylistItemsStarted = false;
 
     QString key;
 
@@ -672,6 +692,8 @@ void ITunesFeature::parsePlaylist(QXmlStreamReader &xml, QSqlQuery &query_insert
                 }
 
                 if (key == "Playlist Items") {
+                    isPlaylistItemsStarted = true;
+                    
                     //if the playlist is prebuild don't hit the database
                     if (isSystemPlaylist) continue;
                     query_insert_to_playlists.bindValue(":id", playlist_id);
@@ -684,9 +706,7 @@ void ITunesFeature::parsePlaylist(QXmlStreamReader &xml, QSqlQuery &query_insert
                         return;
                     }
                     //append the playlist to the child model
-                    TreeItem *item = new TreeItem(playlistname, playlistname, this, root);
-                    root->appendChild(item);
-
+                    root->appendChild(playlistname);
                 }
                 // When processing playlist entries, playlist name and id have
                 // already been processed and persisted
@@ -715,6 +735,10 @@ void ITunesFeature::parsePlaylist(QXmlStreamReader &xml, QSqlQuery &query_insert
                 //qDebug() << "exit playlist";
                 break;
             }
+            if (xml.name() == "dict" && !isPlaylistItemsStarted){
+                // Some playlists can be empty, so we need to exit.
+                break;
+            }
         }
     }
 }
@@ -734,9 +758,9 @@ void ITunesFeature::clearTable(QString table_name) {
 }
 
 void ITunesFeature::onTrackCollectionLoaded() {
-    TreeItem* root = m_future.result();
+    std::unique_ptr<TreeItem> root(m_future.result());
     if (root) {
-        m_childModel.setRootItem(root);
+        m_childModel.setRootItem(std::move(root));
 
         // Tell the rhythmbox track source that it should re-build its index.
         m_trackSource->buildIndex();
