@@ -5,8 +5,8 @@
 const char* kNegatePrefix = "-";
 const char* kFuzzyPrefix = "~";
 
-SearchQueryParser::SearchQueryParser(QSqlDatabase& database)
-        : m_database(database) {
+SearchQueryParser::SearchQueryParser(TrackCollection* pTrackCollection)
+    : m_pTrackCollection(pTrackCollection) {
     m_textFilters << "artist"
                   << "album_artist"
                   << "album"
@@ -15,7 +15,8 @@ SearchQueryParser::SearchQueryParser(QSqlDatabase& database)
                   << "composer"
                   << "grouping"
                   << "comment"
-                  << "location";
+                  << "location"
+                  << "crate";
     m_numericFilters << "year"
                      << "track"
                      << "bpm"
@@ -23,7 +24,11 @@ SearchQueryParser::SearchQueryParser(QSqlDatabase& database)
                      << "rating"
                      << "bitrate";
     m_specialFilters << "key"
-                     << "duration";
+                     << "duration"
+                     << "added"
+                     << "dateadded"
+                     << "datetime_added"
+                     << "date_added";
 
     m_fieldToSqlColumns["artist"] << "artist" << "album_artist";
     m_fieldToSqlColumns["album_artist"] << "album_artist";
@@ -43,6 +48,7 @@ SearchQueryParser::SearchQueryParser(QSqlDatabase& database)
     m_fieldToSqlColumns["played"] << "timesplayed";
     m_fieldToSqlColumns["rating"] << "rating";
     m_fieldToSqlColumns["location"] << "location";
+    m_fieldToSqlColumns["datetime_added"] << "datetime_added";
 
     m_allFilters.append(m_textFilters);
     m_allFilters.append(m_numericFilters);
@@ -109,97 +115,94 @@ void SearchQueryParser::parseTokens(QStringList tokens,
             continue;
         }
 
+        bool negate = token.startsWith(kNegatePrefix);
+        std::unique_ptr<QueryNode> pNode;
+
         if (m_fuzzyMatcher.indexIn(token) != -1) {
             // TODO(XXX): implement this feature.
         } else if (m_textFilterMatcher.indexIn(token) != -1) {
-            bool negate = token.startsWith(kNegatePrefix);
             QString field = m_textFilterMatcher.cap(1);
             QString argument = getTextArgument(
                 m_textFilterMatcher.cap(2), &tokens).trimmed();
 
             if (!argument.isEmpty()) {
-                QueryNode* pNode = new TextFilterNode(
-                    m_database, m_fieldToSqlColumns[field], argument);
-                if (negate) {
-                    pNode = new NotNode(pNode);
+                if (field == "crate") {
+                    pNode = std::make_unique<CrateFilterNode>(
+                          &m_pTrackCollection->crates(), argument);
+                } else {
+                    pNode = std::make_unique<TextFilterNode>(
+                          m_pTrackCollection->database(), m_fieldToSqlColumns[field], argument);
                 }
-                pQuery->addNode(pNode);
             }
         } else if (m_numericFilterMatcher.indexIn(token) != -1) {
-            bool negate = token.startsWith(kNegatePrefix);
             QString field = m_numericFilterMatcher.cap(1);
             QString argument = getTextArgument(
                 m_numericFilterMatcher.cap(2), &tokens).trimmed();
 
             if (!argument.isEmpty()) {
-                QueryNode* pNode = new NumericFilterNode(
-                    m_fieldToSqlColumns[field], argument);
-                if (negate) {
-                    pNode = new NotNode(pNode);
-                }
-                pQuery->addNode(pNode);
+                pNode = std::make_unique<NumericFilterNode>(
+                     m_fieldToSqlColumns[field], argument);
             }
         } else if (m_specialFilterMatcher.indexIn(token) != -1) {
-            bool negate = token.startsWith(kNegatePrefix);
             bool fuzzy = token.startsWith(kFuzzyPrefix);
             QString field = m_specialFilterMatcher.cap(1);
             QString argument = getTextArgument(
                 m_specialFilterMatcher.cap(2), &tokens).trimmed();
-            QueryNode* pNode = NULL;
             if (!argument.isEmpty()) {
                 if (field == "key") {
                     mixxx::track::io::key::ChromaticKey key =
                             KeyUtils::guessKeyFromText(argument);
                     if (key == mixxx::track::io::key::INVALID) {
-                        pNode = new TextFilterNode(
-                            m_database, m_fieldToSqlColumns[field], argument);
+                        pNode = std::make_unique<TextFilterNode>(
+                                m_pTrackCollection->database(), m_fieldToSqlColumns[field], argument);
                     } else {
-                        pNode = new KeyFilterNode(key, fuzzy);
+                        pNode = std::make_unique<KeyFilterNode>(key, fuzzy);
                     }
                 } else if (field == "duration") {
-                    pNode = new DurationFilterNode(m_fieldToSqlColumns[field],
-                                                   argument);
+                    pNode = std::make_unique<DurationFilterNode>(
+                            m_fieldToSqlColumns[field], argument);
+                } else if (field == "date_added" ||
+                           field == "datetime_added" ||
+                           field == "added" ||
+                           field == "dateadded") {
+                    field = "datetime_added";
+                    pNode = std::make_unique<TextFilterNode>(
+                        m_pTrackCollection->database(), m_fieldToSqlColumns[field], argument);
                 }
             }
-            if (pNode != NULL) {
-                if (negate) {
-                    pNode = new NotNode(pNode);
-                }
-                pQuery->addNode(pNode);
-            }
         } else {
             // If no advanced search feature matched, treat it as a search term.
-            bool negate = token.startsWith(kNegatePrefix);
             if (negate) {
                 token = token.mid(1);
             }
-
             // Don't trigger on a lone minus sign.
             if (!token.isEmpty()) {
-                QueryNode* pNode = new TextFilterNode(
-                    m_database, searchColumns, token);
-                if (negate) {
-                    pNode = new NotNode(pNode);
-                }
-                pQuery->addNode(pNode);
+                pNode = std::make_unique<TextFilterNode>(
+                                m_pTrackCollection->database(), searchColumns, token);
             }
         }
+        if (pNode) {
+            if (negate) {
+                pNode = std::make_unique<NotNode>(std::move(pNode));
+            }
+            pQuery->addNode(std::move(pNode));
+        }
     }
 }
 
-QueryNode* SearchQueryParser::parseQuery(const QString& query,
+std::unique_ptr<QueryNode> SearchQueryParser::parseQuery(const QString& query,
                                          const QStringList& searchColumns,
                                          const QString& extraFilter) const {
-    AndNode* pQuery = new AndNode();
+    auto pQuery(std::make_unique<AndNode>());
 
     if (!extraFilter.isEmpty()) {
-        pQuery->addNode(new SqlNode(extraFilter));
+        pQuery->addNode(std::make_unique<SqlNode>(extraFilter));
     }
 
     if (!query.isEmpty()) {
         QStringList tokens = query.split(" ");
-        parseTokens(tokens, searchColumns, pQuery);
+        parseTokens(tokens, searchColumns, pQuery.get());
     }
 
-    return pQuery;
+    return std::move(pQuery);
 }
