@@ -30,11 +30,14 @@ void Tooltips::addStandardTooltips() {
     QString leftClick = tr("Left-click");
     QString rightClick = tr("Right-click");
     QString scrollWheel = tr("Scroll-wheel");
+    QString loopActive = "(" + tr("loop active") + ")";
+    QString loopInactive = "(" + tr("loop inactive") + ")";
+    QString effectsWithinUnit = tr("Effects within the chain must be enabled to hear them.");
 
     add("waveform_overview")
             << tr("Waveform Overview")
             << tr("Shows information about the track currently loaded in this channel.")
-            << tr("Jump around in the track by clicking somewhere on the waveform.")
+            << tr("Jump around in the track by clicking anywhere on the waveform.")
             << dropTracksHere;
 
     QString scratchMouse = tr("Use the mouse to scratch, spin-back or throw tracks.");
@@ -60,6 +63,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Spinning Vinyl")
             << tr("Rotates during playback and shows the position of a track.")
             << scratchMouse
+            << tr("Right click to show cover art of loaded track.")
             << dropTracksHere
             << tr("If Vinyl control is enabled, displays time-coded vinyl signal quality (see Preferences -> Vinyl Control).");
 
@@ -162,6 +166,11 @@ void Tooltips::addStandardTooltips() {
             << tr("Adjusts the master output gain.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
+    add("booth_gain")
+            << tr("Booth Gain")
+            << tr("Adjusts the booth output gain.")
+            << QString("%1: %2").arg(rightClick, resetToDefault);
+
     add("crossfader")
             << tr("Crossfader")
             << tr("Determines the master output by fading between the left and right channels.")
@@ -186,7 +195,7 @@ void Tooltips::addStandardTooltips() {
 
     add("headMix")
             << tr("Headphone Mix")
-            << tr("Controls what you hear on the headphone output.")
+            << tr("Crossfades the headphone output between the master mix and cueing (PFL or Pre-Fader Listening) signal.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
     add("headSplit")
@@ -195,12 +204,10 @@ void Tooltips::addStandardTooltips() {
                   "plays in the left channel.")
             << tr("Adjust the Headphone Mix so in the left channel is not the pure cueing signal.");
 
-    // Note, this is used for samplers and microphone only currently (that's why
-    // center is the default).
     add("orientation")
-            << tr("Mix Orientation")
-            << tr("Set the channel's mix orientation.")
-            << tr("Either to the left side of crossfader, to the right side or to the center (default).");
+            << tr("Crossfader Orientation")
+            << tr("Set the channel's crossfader orientation.")
+            << tr("Either to the left side of crossfader, to the right side or to the center (unaffected by crossfader)");
 
     add("show_microphone")
             << tr("Microphone")
@@ -232,8 +239,8 @@ void Tooltips::addStandardTooltips() {
             << tr("Show or hide the track library.");
 
     add("show_effects")
-            << tr("Show Effect Rack")
-            << tr("Show or hide the effect rack.");
+            << tr("Show Effects")
+            << tr("Show or hide the effects.");
 
     add("maximize_library")
             << tr("Maximize Library")
@@ -258,6 +265,19 @@ void Tooltips::addStandardTooltips() {
             << tr("Hold-to-talk or short click for latching to")
             << tr("mix microphone input into the master output.");
 
+    add("talkover_duck_mode")
+            << tr("Microphone Talkover Mode")
+            << tr("Off: Do not reduce music volume")
+            << tr("Auto: Automatically reduce music volume when microphones are in use. Adjust the amount the music volume is reduced with the Strength knob.")
+            << tr("Manual: Reduce music volume by a fixed amount set by the Strength knob.");
+
+    add("talkover_duck_strength")
+            << tr("Microphone Talkover Ducking Strength")
+            << tr("Behavior depends on Microphone Talkover Mode:")
+            << tr("Off: Does nothing")
+            << tr("Auto: Sets how much to reduce the music volume when microphones are in use.")
+            << tr("Manual: Sets how much to reduce the music volume, regardless of volume of microphone inputs.");
+
     QString changeAmount = tr("Change the step-size in the Preferences -> Interface menu.");
     add("rate_perm_up_rate_perm_up_small")
             << tr("Raise Pitch")
@@ -364,9 +384,9 @@ void Tooltips::addStandardTooltips() {
             << tr("Show/hide the spinning vinyl section.");
 
     add("keylock")
-            << tr("Key-Lock")
+            << tr("Keylock")
             << tr("Prevents the pitch from changing when the rate changes.")
-            << tr("Toggling key-lock during playback may result in a momentary audio glitch.");
+            << tr("Toggling keylock during playback may result in a momentary audio glitch.");
 
     // Used in cue/hotcue/loop tooltips below.
     QString quantizeSnap = tr("If quantize is enabled, snaps to the nearest beat.");
@@ -389,7 +409,7 @@ void Tooltips::addStandardTooltips() {
             << QString("%1: %2").arg(rightClick, tr("Jumps to the beginning of the track and stops."));
 
     // Currently used for decks
-    QString cueSet = tr("Places a cue-point at the current position on the waveform.");
+    QString cueSet = tr("Places a cue point at the current position on the waveform.");
     add("play_cue_set")
             << tr("Play/Pause")
             << QString("%1: %2").arg(leftClick, tr("Plays or pauses the track."))
@@ -399,11 +419,12 @@ void Tooltips::addStandardTooltips() {
     QString whileStopped = tr("(while stopped)");
     add("cue_default_cue_gotoandstop")
             << tr("Cue")
-            << QString("%1 %2: %3").arg(leftClick, whilePlaying, tr("Stops track at cue point."))
-            << QString("%1 %2: %3").arg(leftClick, whileStopped, tr("Set cue point (Pioneer/Mixxx mode) OR preview from it (Denon mode)."))
+            << QString("%1 %2: %3").arg(leftClick, whilePlaying, tr("Stops track at cue point, OR go to cue point and play after release (CUP mode)."))
+            << QString("%1 %2: %3").arg(leftClick, whileStopped, tr("Set cue point (Pioneer/Mixxx/Numark mode), set cue point and play after release (CUP mode) "
+                                                                    "OR preview from it (Denon mode)."))
             << tr("Hint: Change the default cue mode in Preferences -> Interface.")
             << quantizeSnap
-            << QString("%1: %2").arg(rightClick, tr("Seeks the track to the cue-point and stops."));
+            << QString("%1: %2").arg(rightClick, tr("Seeks the track to the cue point and stops."));
 
     add("pfl")
             << tr("Headphone")
@@ -457,7 +478,7 @@ void Tooltips::addStandardTooltips() {
 
     add("rate")
             << tr("Speed Control")
-            << tr("Changes the track playback speed (affects both the tempo and the pitch). If key-lock is enabled, only the tempo is affected.")
+            << tr("Changes the track playback speed (affects both the tempo and the pitch). If keylock is enabled, only the tempo is affected.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
     add("pitch")
@@ -507,11 +528,19 @@ void Tooltips::addStandardTooltips() {
             << tr("Record Mix")
             << tr("Toggle mix recording.");
 
-    add("shoutcast_enabled")
-            << tr("Enable Live Broadcasting")
-            << tr("Stream your mix over the Internet.")
-            << tr("Provides visual feedback for Live Broadcasting status:")
-            << tr("disabled, connecting, connected, failure.");
+    // Status displays and toggle buttons
+    add("recording_duration")
+            << tr("Recording Duration")
+            << tr("Displays the duration of the running recording.");
+
+    // For legacy reasons also add tooltips for "shoutcast_enabled".
+    for (const char* key : {"shoutcast_enabled", "broadcast_enabled"}) {
+        add(key)
+                << tr("Enable Live Broadcasting")
+                << tr("Stream your mix over the Internet.")
+                << tr("Provides visual feedback for Live Broadcasting status:")
+                << tr("disabled, connecting, connected, failure.");
+    }
 
     add("passthrough_enabled")
             << tr("Enable Passthrough")
@@ -544,13 +573,21 @@ void Tooltips::addStandardTooltips() {
 
     add("loop_in")
             << tr("Loop-In Marker")
-            << tr("Sets the deck loop-in position to the current play position.")
-            << quantizeSnap;
+            << QString("%1: %2").arg(leftClick + " " + loopInactive,
+                      tr("Sets the track Loop-In Marker to the current play position."))
+            << quantizeSnap
+            << QString("%1: %2").arg(leftClick + " " + loopActive,
+                      tr("Press and hold to move Loop-In Marker."))
+            << QString("%1: %2").arg(rightClick, tr("Jump to Loop-In Marker."));
 
     add("loop_out")
             << tr("Loop-Out Marker")
-            << tr("Sets the deck loop-out position to the current play position.")
-            << quantizeSnap;
+            << QString("%1: %2").arg(leftClick + " " + loopInactive,
+                      tr("Sets the track Loop-Out Marker to the current play position."))
+            << quantizeSnap
+            << QString("%1: %2").arg(leftClick + " " + loopActive,
+                      tr("Press and hold to move Loop-Out Marker."))
+            << QString("%1: %2").arg(rightClick, tr("Jump to Loop-Out Marker."));
 
     add("loop_halve")
             << tr("Loop Halve")
@@ -561,31 +598,54 @@ void Tooltips::addStandardTooltips() {
             << tr("Loop Double")
             << tr("Doubles the current loop's length by moving the end marker.");
 
+    add("beatloop_size")
+            << tr("Beatloop Size")
+            << tr("Select the size of the loop in beats to set with the Beatloop button.")
+            << tr("Changing this resizes the loop if the loop already matches this size.");
+
+    add("beatloop_halve")
+            << tr("Halve the size of an existing beatloop, or halve the size of the next beatloop set with the Beatloop button.");
+
+    add("beatloop_double")
+            << tr("Double the size of an existing beatloop, or double the size of the next beatloop set with the Beatloop button.");
+
     //beatloop and beatlooproll
-    add("beatloop")
+    add("beatloop_activate")
             << tr("Beatloop")
-            << QString("%1: %2").arg(leftClick, tr("Setup a loop over the set number of beats."))
+            << QString("%1: %2").arg(leftClick, tr("Start a loop over the set number of beats."))
             << quantizeSnap
-            << QString("%1: %2").arg(rightClick, tr("Temporarily setup a rolling loop over the set number of beats."))
+            << QString("%1: %2").arg(rightClick, tr("Temporarily enable a rolling loop over the set number of beats."))
             << tr("Playback will resume where the track would have been if it had not entered the loop.");
 
-    add("beatjump")
-            << tr("Beatjump")
-            << QString("%1: %2").arg(leftClick, tr("Jump forward or backward by the set number of beats."));
+    add("beatjump_size")
+            << tr("Beatjump/Loop Move Size")
+            << tr("Select the number of beats to jump or move the loop with the Beatjump Forward/Backward buttons.");
 
-    add("loop_move")
-            << tr("Loop Move")
-            << QString("%1: %2").arg(leftClick, tr("Adjust the loop in and out points by the set number of beats."));
+    add("beatjump_forward")
+            << tr("Beatjump Forward")
+            << QString("%1: %2").arg(leftClick + " " + loopInactive, tr("Jump forward by the set number of beats."))
+            << QString("%1: %2").arg(leftClick + " " + loopActive, tr("Move the loop forward by the set number of beats."))
+            << QString("%1: %2").arg(rightClick + " " + loopInactive, tr("Jump forward by 1 beat."))
+            << QString("%1: %2").arg(rightClick + " " + loopActive, tr("Move the loop forward by 1 beat."));
+
+    add("beatjump_backward")
+            << tr("Beatjump Backward")
+            << QString("%1: %2").arg(leftClick + " " + loopInactive, tr("Jump backward by the set number of beats."))
+            << QString("%1: %2").arg(leftClick + " " + loopActive, tr("Move the loop backward by the set number of beats."))
+            << QString("%1: %2").arg(rightClick + " " + loopInactive, tr("Jump backward by 1 beat."))
+            << QString("%1: %2").arg(rightClick + " " + loopActive, tr("Move the loop backward by 1 beat."));
 
     add("loop_exit")
             << tr("Loop Exit")
             << tr("Turns the current loop off.")
             << tr("Works only if Loop-In and Loop-Out marker are set.");
 
-    add("reloop_exit")
-            << tr("Reloop/Exit")
-            << tr("Toggles the current loop on or off.")
-            << tr("Works only if Loop-In and Loop-Out marker are set.");
+    add("reloop_toggle")
+            << tr("Reloop")
+            << QString("%1: %2").arg(leftClick, tr("Toggles the current loop on or off."))
+            << tr("If the loop is ahead of the current position, looping will start when the loop is reached.")
+            << tr("Works only if Loop-In and Loop-Out Marker are set.")
+            << QString("%1: %2").arg(rightClick, tr("Enable loop, jump to Loop-In Marker, and stop playback."));
 
     add("slip_mode")
             << tr("Slip Mode")
@@ -594,8 +654,8 @@ void Tooltips::addStandardTooltips() {
 
     add("track_time")
             << tr("Track Time")
-            << tr("Displays the elapsed or remaining time of the track loaded.")
-            << tr("Click to toggle between time elapsed/remaining time.");
+            << tr("Displays the elapsed and/or remaining time of the track loaded.")
+            << tr("Click to toggle between time elapsed/remaining time/both.");
 
     add("track_duration")
             << tr("Track Duration")
@@ -659,18 +719,21 @@ void Tooltips::addStandardTooltips() {
             << tr("Clear Unit")
             << tr("Clear effect unit.");
 
+    add("EffectUnit_show_parameters")
+            << tr("Show Effect Parameters")
+            << tr("Show/hide parameters for effects in this unit.");
+
     add("EffectUnit_enabled")
             << tr("Toggle Unit")
-            << tr("Enable or disable effect processing.");
-
+            << tr("Enable or disable this whole effect unit.");
     add("EffectUnit_mix")
             << tr("Dry/Wet")
-            << tr("Adjust the balance between the original (dry) and processed (wet) signal.")
+            << tr("Adjust the balance between the original (dry) and processed (wet) signal for the whole effect unit.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
     add("EffectUnit_super1")
             << tr("Super Knob")
-            << tr("Super Knob (control linked effect parameters).")
+            << tr("Controls the Meta Knob of all effects in this unit together.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
     add("EffectUnit_insertion_type")
@@ -679,19 +742,60 @@ void Tooltips::addStandardTooltips() {
 
     add("EffectUnit_next_chain")
             << tr("Next Chain")
-            << tr("Next chain preset.");
+            << tr("Load next effect chain preset into this effect unit.");
 
     add("EffectUnit_prev_chain")
             << tr("Previous Chain")
-            << tr("Previous chain preset.");
+            << tr("Load previous effect chain preset into this effect unit.");
 
     add("EffectUnit_chain_selector")
             << tr("Next/Previous Chain")
-            << tr("Next or previous chain preset.");
+            << tr("Load next or previous effect chain preset into this effect unit.");
 
     add("EffectUnit_group_enabled")
-            << tr("Assign Effect Chain")
-            << tr("Assign effect unit to the channel output.");
+            << tr("Assign Effect Unit")
+            << tr("Assign this effect unit to the channel output.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_headphones_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route the headphone channel through this effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_master_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route the master mix through this effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_BusLeft_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route the left crossfader bus through this effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_BusRight_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route the right crossfader bus through this effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_deck_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route this deck through the indicated effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_sampler_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route this sampler through the indicated effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_microphone_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route this microphone through the indicated effect unit.")
+            << effectsWithinUnit;
+
+    add("EffectUnit_auxiliary_enabled")
+            << tr("Assign Effect Unit")
+            << tr("Route this auxiliary input through the indicated effect unit.")
+            << effectsWithinUnit;
 
     // Effect Slot Controls
     add("EffectSlot_clear")
@@ -699,8 +803,8 @@ void Tooltips::addStandardTooltips() {
             << tr("Clear the current effect.");
 
     add("EffectSlot_enabled")
-            << tr("Toggle")
-            << tr("Toggle the current effect.");
+            << tr("Enable Effect")
+            << tr("The effect unit must also be assigned to a deck or other sound source to hear the effect.");
 
     add("EffectSlot_next_effect")
             << tr("Next")
@@ -714,14 +818,34 @@ void Tooltips::addStandardTooltips() {
             << tr("Next or Previous")
             << tr("Switch to either the next or previous effect.");
 
+    add("EffectSlot_metaknob")
+            << tr("Meta Knob")
+            << tr("Controls linked parameters of this effect")
+            << QString("%1: %2").arg(rightClick, resetToDefault);
+
+    add("EffectSlot_focus")
+            << tr("Effect Focus Button")
+            << QString("%1: %2").arg(leftClick, tr("Focuses this effect."))
+            << QString("%1: %2").arg(rightClick, tr("Unfocuses this effect."))
+            << tr("Refer to the web page on the Mixxx wiki for your controller for more information.");
+
     add("EffectSlot_parameter")
             << tr("Effect Parameter")
             << tr("Adjusts a parameter of the effect.")
             << QString("%1: %2").arg(rightClick, resetToDefault);
 
     add("EffectSlot_parameter_link_type")
-            << tr("Super Knob Mode")
-            << tr("Set how linked effect parameters change when turning the Super Knob.");
+            << tr("Meta Knob Link")
+            << tr("Set how this parameter is linked to the effect's Meta Knob.")
+            << tr("Inactive: parameter not linked")
+            << tr("Active: parameter moves with Meta Knob")
+            << tr("Left side active: parameter moves with left half of Meta Knob turn")
+            << tr("Right side active: parameter moves with right half of Meta Knob turn")
+            << tr("Left and right side active: parameter moves across range with half of Meta Knob turn and back with the other half");
+
+    add("EffectSlot_parameter_inversion")
+            << tr("Meta Knob Link Inversion")
+            << tr("Inverts the direction this parameter moves when turning the effect's Meta Knob.");
 
     add("EffectSlot_button_parameter")
             << tr("Equalizer Parameter Kill")
@@ -751,4 +875,17 @@ void Tooltips::addStandardTooltips() {
             << tr("Equalizer Parameter Kill")
             << tr("Holds the gain of the EQ to zero while active.")
             << eqKillLatch;
+
+    add("skin_settings")
+            << tr("Skin Settings Menu")
+            << tr("Show/hide skin settings menu");
+
+    // Sampler Bank Controls
+    add("SaveSamplerBank")
+            << tr("Save Sampler Bank")
+            << tr("Save the collection of samples loaded in the samplers.");
+    
+    add("LoadSamplerBank")
+            << tr("Load Sampler Bank")
+            << tr("Load a previously saved collection of samples into the samplers.");
 }
