@@ -7,33 +7,33 @@
 #include <QPainter>
 #include <QMimeData>
 
-#include "controlobject.h"
-#include "controlobjectslave.h"
-#include "trackinfoobject.h"
+#include "control/controlobject.h"
+#include "control/controlproxy.h"
+#include "track/track.h"
 #include "waveform/widgets/waveformwidgetabstract.h"
 #include "widget/wwaveformviewer.h"
 #include "waveform/waveformwidgetfactory.h"
 #include "util/dnd.h"
 #include "util/math.h"
 
-WWaveformViewer::WWaveformViewer(const char *group, ConfigObject<ConfigValue>* pConfig, QWidget * parent)
+WWaveformViewer::WWaveformViewer(const char *group, UserSettingsPointer pConfig, QWidget * parent)
         : WWidget(parent),
           m_pGroup(group),
           m_pConfig(pConfig),
           m_zoomZoneWidth(20),
           m_bScratching(false),
           m_bBending(false),
-          m_waveformWidget(NULL) {
+          m_waveformWidget(nullptr) {
     setAcceptDrops(true);
 
-    m_pZoom = new ControlObjectSlave(group, "waveform_zoom", this);
+    m_pZoom = new ControlProxy(group, "waveform_zoom", this);
     m_pZoom->connectValueChanged(SLOT(onZoomChange(double)));
 
-    m_pScratchPositionEnable = new ControlObjectSlave(
+    m_pScratchPositionEnable = new ControlProxy(
             group, "scratch_position_enable", this);
-    m_pScratchPosition = new ControlObjectSlave(
+    m_pScratchPosition = new ControlProxy(
             group, "scratch_position", this);
-    m_pWheel = new ControlObjectSlave(
+    m_pWheel = new ControlProxy(
             group, "wheel", this);
 
     setAttribute(Qt::WA_OpaquePaintEvent);
@@ -43,8 +43,7 @@ WWaveformViewer::~WWaveformViewer() {
     //qDebug() << "~WWaveformViewer";
 }
 
-void WWaveformViewer::setup(QDomNode node, const SkinContext& context) {
-    Q_UNUSED(context);
+void WWaveformViewer::setup(const QDomNode& node, const SkinContext& context) {
     if (m_waveformWidget) {
         m_waveformWidget->setup(node, context);
     }
@@ -57,9 +56,13 @@ void WWaveformViewer::resizeEvent(QResizeEvent* /*event*/) {
 }
 
 void WWaveformViewer::mousePressEvent(QMouseEvent* event) {
+    if (!m_waveformWidget) {
+        return;
+    }
+
     m_mouseAnchor = event->pos();
 
-    if (event->button() == Qt::LeftButton && m_waveformWidget) {
+    if (event->button() == Qt::LeftButton) {
         // If we are pitch-bending then disable and reset because the two
         // shouldn't be used at once.
         if (m_bBending) {
@@ -67,8 +70,10 @@ void WWaveformViewer::mousePressEvent(QMouseEvent* event) {
             m_bBending = false;
         }
         m_bScratching = true;
+        int eventPosValue = m_waveformWidget->getOrientation() == Qt::Horizontal ?
+                    event->pos().x() : event->pos().y();
         double audioSamplePerPixel = m_waveformWidget->getAudioSamplePerPixel();
-        double targetPosition = -1.0 * event->pos().x() * audioSamplePerPixel * 2;
+        double targetPosition = -1.0 * eventPosValue * audioSamplePerPixel * 2;
         m_pScratchPosition->set(targetPosition);
         m_pScratchPositionEnable->slotSet(1.0);
     } else if (event->button() == Qt::RightButton) {
@@ -87,15 +92,23 @@ void WWaveformViewer::mousePressEvent(QMouseEvent* event) {
 }
 
 void WWaveformViewer::mouseMoveEvent(QMouseEvent* event) {
+    if (!m_waveformWidget) {
+        return;
+    }
+
     // Only send signals for mouse moving if the left button is pressed
-    if (m_bScratching && m_waveformWidget) {
+    if (m_bScratching) {
+        int eventPosValue = m_waveformWidget->getOrientation() == Qt::Horizontal ?
+                    event->pos().x() : event->pos().y();
         // Adjusts for one-to-one movement.
         double audioSamplePerPixel = m_waveformWidget->getAudioSamplePerPixel();
-        double targetPosition = -1.0 * event->pos().x() * audioSamplePerPixel * 2;
+        double targetPosition = -1.0 * eventPosValue * audioSamplePerPixel * 2;
         //qDebug() << "Target:" << targetPosition;
         m_pScratchPosition->set(targetPosition);
     } else if (m_bBending) {
         QPoint diff = event->pos() - m_mouseAnchor;
+        int diffValue = m_waveformWidget->getOrientation() == Qt::Horizontal ?
+                    diff.x() : diff.y();
         // Start at the middle of [0.0, 1.0], and emit values based on how far
         // the mouse has traveled horizontally. Note, for legacy (MIDI) reasons,
         // this is tuned to 127.
@@ -103,7 +116,7 @@ void WWaveformViewer::mouseMoveEvent(QMouseEvent* event) {
         // control since we manually connect it in LegacySkinParser regardless
         // of whether the skin specifies it. See ControlTTRotaryBehavior to see
         // where this value is handled.
-        double v = 0.5 + (diff.x() / 1270.0);
+        double v = 0.5 + (diffValue / 1270.0);
         // clamp to [0.0, 1.0]
         v = math_clamp(v, 0.0, 1.0);
         m_pWheel->setParameter(v);
@@ -163,13 +176,15 @@ void WWaveformViewer::dropEvent(QDropEvent* event) {
     event->ignore();
 }
 
-void WWaveformViewer::onTrackLoaded(TrackPointer track) {
+void WWaveformViewer::slotTrackLoaded(TrackPointer track) {
     if (m_waveformWidget) {
         m_waveformWidget->setTrack(track);
     }
 }
 
-void WWaveformViewer::onTrackUnloaded(TrackPointer /*track*/) {
+void WWaveformViewer::slotLoadingTrack(TrackPointer pNewTrack, TrackPointer pOldTrack) {
+    Q_UNUSED(pNewTrack);
+    Q_UNUSED(pOldTrack);
     if (m_waveformWidget) {
         m_waveformWidget->setTrack(TrackPointer());
     }
@@ -200,6 +215,10 @@ void WWaveformViewer::setZoom(int zoom) {
     }
 }
 
+void WWaveformViewer::setDisplayBeatGrid(bool set) {
+    m_waveformWidget->setDisplayBeatGrid(set);
+}
+
 void WWaveformViewer::setWaveformWidget(WaveformWidgetAbstract* waveformWidget) {
     if (m_waveformWidget) {
         QWidget* pWidget = m_waveformWidget->getWidget();
